#################################
# Selection of policy variables #
#################################

#library(fixest)
library(tidyverse)
library(data.table)
library(fixest)
library(glmnet)
library(penppml)
library(openxlsx)

# Directories
table_fn<-"./E. Output/Tables/"
data_dir<-"./A. Data/"
output_dir<-"./E. Output/"
function_dir<-"./F. Functions/"

##############
# PARAMETERS #
##############

# PRODUCT/SECTOR
PROD="CHS"

# REFERENCE COUNTRY
REF="AUS"

# WRITE OUTPUT FILES 
# turn off if checking/changing code so as not to overwrite
WRITE=FALSE

#############
# LOAD DATA #
#############

# All data includes bilateral trade flows that are zero in every year in the panel
df<-readRDS(file=paste0(data_dir,PROD,"_mdl_data.rds"))

# Filter the data so that data is not missing in the year that we estimate the simulation model
# First, drop any rows for which there is no trade (in sum) for the simulation year
zero_trade<-filter(df,Year==max(df$Year),DestinationYearTrade==0|OriginYearTrade==0)
drop_isos<-unique(zero_trade$Origin)
df<-filter(df,!Destination%in%all_of(drop_isos),!Origin%in%all_of(drop_isos))
df<-droplevels(df)
# Drop any countries for which domestic trade is zero 
zero_domestic<-filter(df,External==0&TradeMillions==0,Year==max(df$Year))
zero_domestic_isos<-unique(zero_domestic$Origin)
df<-filter(df,!Destination%in%all_of(zero_domestic_isos),!Origin%in%all_of(zero_domestic_isos))
df<-droplevels(df)
# Idiosyncratic... EXP_SUB has NAs. Convert to zero
df[is.na(df)]<-0


####################
# Create variables #
####################

# Add an EU interaction term
df<-df%>%
  mutate(INV_X_EU = INV_X*EUN,
         INV_X_EU_L1 = INV_X_L1*EUN,
         INV_X_EU_L2 = INV_X_L2*EUN,
         INV_X_EU_L3 = INV_X_L3*EUN,
         DEV_X_EU = DEV_X*EUN,
         DEV_X_EU_L1 = DEV_X_L1*EUN,
         DEV_X_EU_L2 = DEV_X_L2*EUN,
         DEV_X_EU_L3 = DEV_X_L3*EUN,
         DCP_X_EU = DCP_X*EUN,
         DCP_X_EU_L1 = DCP_X_L1*EUN,
         DCP_X_EU_L2 = DCP_X_L2*EUN,
         DCP_X_EU_L3 = DCP_X_L3*EUN,
         DRY_X_EU = DRY_X*EUN,
         DRY_X_EU_L1 = DRY_X_L1*EUN,
         DRY_X_EU_L2 = DRY_X_L2*EUN,
         DRY_X_EU_L3 = DRY_X_L3*EUN,
         LIV_X_EU = LIV_X*EUN,
         LIV_X_EU_L1 = LIV_X_L1*EUN,
         LIV_X_EU_L2 = LIV_X_L2*EUN,
         LIV_X_EU_L3 = LIV_X_L3*EUN,
         CRP_X_EU = CRP_X*EUN,
         CRP_X_EU_L1 = CRP_X_L1*EUN,
         CRP_X_EU_L2 = CRP_X_L2*EUN,
         CRP_X_EU_L3 = CRP_X_L3*EUN,
         OTH_X_EU = OTH_X*EUN,
         OTH_X_EU_L1 = OTH_X_L1*EUN,
         OTH_X_EU_L2 = OTH_X_L2*EUN,
         OTH_X_EU_L3 = OTH_X_L3*EUN)

X.mat<-data.matrix(df[,c("Tariff","Tariff_L1","Tariff_L2","Tariff_L3",
                         "RTA","RTA_L1","RTA_L2","RTA_L3",
                         "EUQ_X",
                         "INV_X","DEV_X","DCP_X","DRY_X","LIV_X","CRP_X","OTH_X", #FDN FARM VARIABLES
                         "INV_X_L1","DEV_X_L1","DCP_X_L1","DRY_X_L1","LIV_X_L1","CRP_X_L1","OTH_X_L1",
                         "INV_X_L2","DEV_X_L2","DCP_X_L2","DRY_X_L2","LIV_X_L2","CRP_X_L2","OTH_X_L2",
                         "INV_X_L3","DEV_X_L3","DCP_X_L3","DRY_X_L3","LIV_X_L3","CRP_X_L3","OTH_X_L3",
                         "INV_X_EU","DEV_X_EU","DCP_X_EU","DRY_X_EU","LIV_X_EU","CRP_X_EU","OTH_X_EU",
                         "INV_X_EU_L1","DEV_X_EU_L1","DCP_X_EU_L1","DRY_X_EU_L1","LIV_X_EU_L1","CRP_X_EU_L1","OTH_X_EU_L1",
                         "INV_X_EU_L2","DEV_X_EU_L2","DCP_X_EU_L2","DRY_X_EU_L2","LIV_X_EU_L2","CRP_X_EU_L2","OTH_X_EU_L2",
                         "INV_X_EU_L3","DEV_X_EU_L3","DCP_X_EU_L3","DRY_X_EU_L3","LIV_X_EU_L3","CRP_X_EU_L3","OTH_X_EU_L3",
                         "TGR_X","TAR_X","TBR_X",
                         "TGR_X_L1","TAR_X_L1","TBR_X_L1",
                         "TGR_X_L2","TAR_X_L2","TBR_X_L2",
                         "TGR_X_L3","TAR_X_L3","TBR_X_L3")])
X.names<-colnames(X.mat)


#################################################################################
# Model selection, following the penalised HDFE methods in Breinlich et al 2021 #
#################################################################################

# This code differs from the methods in Breinlich et al 2021
# in so far as the penalised regression is relaxed lasso.

# The function(s) begin will a lot of input checking
# It then starts an iteration to solve the maximum likelihood of the model. 
# On the first iteration [if/assuming mu is NULL in the inputs]
# an unpenalised high dimensional fixed effects ppml [hdfeppml] is estimated
# to get the first (set of) estimate(s) of mu. Only the fixed effects are estimated

# PRELIMINARIES

# (1) Data frame - base/reference data

# (a) Variables to select
penpmml_vars<-c(c("TradeMillions","Origin","Destination","Year","OriginYear","DestinationYear","Direction"),X.names)

# (b) Construct dataframe
df1<-df%>%
  filter(DirectionTrade>0)%>%
  mutate(OriginYear=factor(as.character(OriginYear)),DestinationYear=factor(as.character(DestinationYear)),
         Direction=factor(Direction))%>%
  dplyr::select(all_of(penpmml_vars))%>%
  ungroup()

# (2) Prepare model input data

# To begin, we need 
# (a) The dependent vector
y = df1$TradeMillions
# (b) The fixed effects
fes<-as.list(df1[,c("OriginYear","DestinationYear","Direction")])
# The x values, non fes, in this case getting all of them
x <- data.matrix(df1[,8:84])
# Initialise coeff vector/matrix
b <- matrix(NA, nrow = ncol(x), ncol = 1) # Initialising
rownames(b) <- colnames(x) # naming
include_x <- 1:ncol(x) # number of vars


# (3) Use cross validation on the 'de-meaned' model/data used in the hdfe estimation

# (a) Estimate the starting value for the weights required, using only (sum of) fixed effects
only_fes <- hdfeppml_int(y, fes = fes, tol = 1e-08, 
                         hdfetol = 1e-04, colcheck_x = FALSE, colcheck_x_fes = FALSE, 
                         mu = NULL, saveX = FALSE, init_z = NULL, verbose = FALSE, 
                         maxiter = 1000, cluster = NULL, vcv = TRUE)

# (b) vars needed for model used for cross-validation of model of policy parameters
# Weights
mu <- only_fes$mu
# Calculate the transformed dependent variable, given the mu (weight) value        
reg_z <- matrix((y - mu)/mu + log(mu))
# Create a matrix from the dependent data matrix, where the fixed effects are non-zero
reg_x <- x[which(mu > 0), ]
# Weighted linear within transform of independent     
z_resid <- collapse::fhdwithin(reg_z, fes, w = mu)
# Weighted linear within transform of dependent
x_resid <- collapse::fhdwithin(reg_x, fes,w = mu)

# (c) Do cross validation, using the transformed model (within transformed to partial out fixed effects)
# This will take a several minutes
penreg.cv=cv.glmnet(x = x_resid, y = z_resid, relax=TRUE,
                    weights = mu/sum(mu), nlambda = 1000, thresh = 1e-12, 
                    family = gaussian(link = "identity"))
plot(penreg.cv)
print(penreg.cv)

# Save a copy of the cv
save(penreg.cv,file=paste0(output_dir,paste0(PROD,"_cross_validation.rda")))

# (d) Candidate lambda values/ranges
LambdaMin=penreg.cv$lambda.min
LambdaSe1=penreg.cv$lambda.1se
LambdaMid=(LambdaMin+LambdaSe1)/2
GammaMin=penreg.cv$relaxed$gamma.min
RelaxLambdaMin=penreg.cv$relaxed$lambda.min
RelaxLambdaSe1=penreg.cv$relaxed$lambda.1se
RelaxLambdaMid=(penreg.cv$relaxed$lambda.min+penreg.cv$relaxed$lambda.1se)/2

# Penalised PPML - via loop to find weights
lambda_range=c(LambdaMin,LambdaMid,LambdaSe1)
lambda_range
#lambda=LambdaMin
ApproxLambdaMid=penreg.cv$lambda[72]


# (e) Estimate model at lambda and gamma min - to maximise policy coverage
# Set lambda and gamma
lambda=RelaxLambdaMin
gamma=GammaMin

#A range for sensitivity testing
Lambda_seq<-seq(from=RelaxLambdaMin,to=RelaxLambdaSe1,by=RelaxLambdaMin)


# (f) Model coefficient estimation, maximimum likelihood loop
# with relaxed lasso for penalisation 
# Loop set up
n <- length(y)
crit <- 1
old_deviance <- 0
iter <- 0
tol = 1e-08

# While loop comparing critical value to preset tolerance
while (crit > tol) {
  # Set counter for new iteration
  iter <- iter + 1
  
  # Add a break to the iterations
  if (iter > 50) {
    message("Exceeded 50 iterations. Break loop and return last model.")
    break
  }	
  
  # On the first iteration... prepartion commences with this, setting up
  # values for the fixed effects (mu) and the transformed (de-meaned) values for
  # the dependent and independent variables - within transformed so that they reflect vaues after fes partialled out
  
  if (iter == 1) {
    # Estimate the weights using only fixed effects
    only_fes <- hdfeppml_int(y, fes = fes, tol = 1e-08, 
                             hdfetol = 1e-04, colcheck_x = FALSE, colcheck_x_fes = FALSE, 
                             mu = NULL, saveX = FALSE, init_z = NULL, verbose = FALSE, 
                             maxiter = 1000, cluster = NULL, vcv = TRUE)
    # Initial guess at weights
    mu <- only_fes$mu
    
    # Calculate the transformed dependent variable, given the fe/mu value        
    z <- (y - mu)/mu + log(mu)
    
    # This appears not to be used, but is explicitly calcuted in the Stata version
    eta <- log(mu)
    
    # Assign the transformed value to a prior value for the next iteration
    last_z <- z
    
    # Convert the z value to a matric for next step(s) estimation
    reg_z <- matrix(z)
    
    # Create a matrix from the dependent data matrix, where the fixed effects are non-zero
    reg_x <- x[which(mu > 0), ]
    
    # Find the number of observations (rows in the transformed depedent variables)
    n <- length(z)
  }
  
  # Subsequent iterations, start with this 
  
  else {
    
    # Store transformed dependent variable
    last_z <- z
    
    # Repeat calculation of number of observations [necessary? just a check?]
    n <- length(mu)
    
    # Recalculate the transformed dependent
    z <- (y - mu)/mu + log(mu)
    
    # Recalculate the matrices needed to re-estimate the (re-)weighted least squares
    reg_z <- matrix(z - last_z[which(mu > 0)] + z_resid[which(mu > 0)])
    reg_x <- x_resid[which(mu > 0), ]
    
    # End of preparatory transformations
  }
  
  # Weighted linear within transform the transformed dependent, given 3-way fixed effects     
  z_resid <- collapse::fhdwithin(reg_z, fes, w = mu)
  
  # Weighted linear within transform the transformed dependent, given 3-way fixed effects
  x_resid <- collapse::fhdwithin(reg_x, fes,w = mu)
  
  # Penalised regression.
  # Note that a number of the inputs shopwn here can be kept at default
  # (i) thresh (ii) penalty.factor (iii) standardize	
  penreg <- glmnet::glmnet(x = x_resid, y = z_resid, gamma=gamma,
                           weights = mu/sum(mu), lambda = lambda, 
                           family = gaussian(link = "identity"))
  
  # The penalised regression's parameters on the transformed (partialled out)
  # dependents and independent variables
  b[include_x] <- penreg$beta
  
  # Residuals from the penalised regression
  residuals <- z_resid - x_resid %*% b[include_x]
  
  # New weights - exponential of the transformed variable less the residuals
  mu <- as.numeric(exp(z - residuals))
  
  # Apply floors and ceilings to the weights
  mu[which(mu < 1e-190)] <- 1e-190
  mu[mu > 1e+190] <- 1e+190
  
  # Calculate the deviance undert this iteration
  temp <- -(y * log(y/mu) - (y - mu))
  temp[which(y == 0)] <- -mu[which(y == 0)]
  deviance <- -2 * sum(temp)/n
  if (deviance < 0) {
    deviance = 0
  }
  
  # Delta in deviance
  delta_deviance <- old_deviance - deviance
  if (!is.na(delta_deviance) & (deviance < 0.1 * delta_deviance)) {
    delta_deviance = deviance
  }
  
  # Critical value 
  denom_crit = max(c(min(c(deviance, old_deviance)), 0.1))
  crit = abs(delta_deviance)/denom_crit
  
  # Store the deviance as the old deviance
  old_deviance <- deviance
  
} # END OF WHILE LOOP 


# Collect results
k <- ncol(matrix(x))
n <- length(y)
select_x <- which(b != 0)
k <- length(select_x)
bic <- deviance + k * log(n)/n

penreg[["beta"]] <- b
penreg[["mu"]] <- mu
penreg[["bic"]] <- bic
penreg[["deviance"]] <- deviance
penreg[["x_resid"]] <- x_resid
penreg[["z_resid"]] <- z_resid

# Extract the coefficients
coefs<-data.frame(name=row.names(penreg$beta),coef=b[,1])
row.names(coefs)<-NULL
write.csv(coefs,paste0(output_dir,PROD,"_relaxed_lasso_lamba_min_coefs.csv"))
policy_coefs=coefs
save(policy_coefs,file=paste0(output_dir,paste0(PROD,"_policy_coefs.rda")))


# Save the model
save(penreg,file=paste0(output_dir,paste0(PROD,"_penalised_ppml_model.rda")))


########### SENSITIVTY TESTS ##########

# For post selection model/info, use this function
# which corrects typos in penppml functions
source(paste0(function_dir,"Corrected_hdfeppml_functions.R"))

out<-list()

for(i in seq_along(Lambda_seq)){
  
    lambda=Lambda_seq[i]
    gamma=0.5
  
    # Loop set up
    n <- length(y)
    crit <- 1
    old_deviance <- 0
    iter <- 0
    tol = 1e-08
    
    # While loop comparing critical value to preset tolerance
    while (crit > tol) {
      # Set counter for new iteration
      iter <- iter + 1
      
      # Add a break to the iterations
      if (iter > 50) {
        message("Exceeded 50 iterations. Break loop and return last model.")
        break
      }	
      
      # On the first iteration... prepartion commences with this, setting up
      # values for the fixed effects (mu) and the transformed (de-meaned) values for
      # the dependent and independent variables - within transformed so that they reflect vaues after fes partialled out
      
      if (iter == 1) {
        # Estimate the weights using only fixed effects
        only_fes <- hdfeppml_int(y, fes = fes, tol = 1e-08, 
                                 hdfetol = 1e-04, colcheck_x = FALSE, colcheck_x_fes = FALSE, 
                                 mu = NULL, saveX = FALSE, init_z = NULL, verbose = FALSE, 
                                 maxiter = 1000, cluster = NULL, vcv = TRUE)
        # Initial guess at weights
        mu <- only_fes$mu
        
        # Calculate the transformed dependent variable, given the fe/mu value        
        z <- (y - mu)/mu + log(mu)
        
        # This appears not to be used, but is explicitly calcuted in the Stata version
        eta <- log(mu)
        
        # Assign the transformed value to a prior value for the next iteration
        last_z <- z
        
        # Convert the z value to a matric for next step(s) estimation
        reg_z <- matrix(z)
        
        # Create a matrix from the dependent data matrix, where the fixed effects are non-zero
        reg_x <- x[which(mu > 0), ]
        
        # Find the number of observations (rows in the transformed depedent variables)
        n <- length(z)
      }
      
      # Subsequent iterations, start with this 
      
      else {
        
        # Store transformed dependent variable
        last_z <- z
        
        # Repeat calculation of number of observations [necessary? just a check?]
        n <- length(mu)
        
        # Recalculate the transformed dependent
        z <- (y - mu)/mu + log(mu)
        
        # Recalculate the matrices needed to re-estimate the (re-)weighted least squares
        reg_z <- matrix(z - last_z[which(mu > 0)] + z_resid[which(mu > 0)])
        reg_x <- x_resid[which(mu > 0), ]
        
        # End of preparatory transformations
      }
      
      # Weighted linear within transform the transformed dependent, given 3-way fixed effects     
      z_resid <- collapse::fhdwithin(reg_z, fes, w = mu)
      
      # Weighted linear within transform the transformed dependent, given 3-way fixed effects
      x_resid <- collapse::fhdwithin(reg_x, fes,w = mu)
      
      # Penalised regression.
      # Note that a number of the inputs shopwn here can be kept at default
      # (i) thresh (ii) penalty.factor (iii) standardize	
      penreg <- glmnet::glmnet(x = x_resid, y = z_resid, gamma=GammaMin,
                               weights = mu/sum(mu), lambda = lambda, 
                               family = gaussian(link = "identity"))
      
      # The penalised regression's parameters on the transformed (partialled out)
      # dependents and independent variables
      b[include_x] <- penreg$beta
      
      # Residuals from the penalised regression
      residuals <- z_resid - x_resid %*% b[include_x]
      
      # New weights - exponential of the transformed variable less the residuals
      mu <- as.numeric(exp(z - residuals))
      
      # Apply floors and ceilings to the weights
      mu[which(mu < 1e-190)] <- 1e-190
      mu[mu > 1e+190] <- 1e+190
      
      # Calculate the deviance undert this iteration
      temp <- -(y * log(y/mu) - (y - mu))
      temp[which(y == 0)] <- -mu[which(y == 0)]
      deviance <- -2 * sum(temp)/n
      if (deviance < 0) {
        deviance = 0
      }
      
      # Delta in deviance
      delta_deviance <- old_deviance - deviance
      if (!is.na(delta_deviance) & (deviance < 0.1 * delta_deviance)) {
        delta_deviance = deviance
      }
      
      # Critical value 
      denom_crit = max(c(min(c(deviance, old_deviance)), 0.1))
      crit = abs(delta_deviance)/denom_crit
      
      # Store the deviance as the old deviance
      old_deviance <- deviance
      
    } # END OF WHILE LOOP 
    
    
    # Collect results
    k <- ncol(matrix(x))
    n <- length(y)
    select_x <- which(b != 0)
    k <- length(select_x)
    bic <- deviance + k * log(n)/n
    
    penreg[["beta"]] <- b
    penreg[["mu"]] <- mu
    penreg[["bic"]] <- bic
    penreg[["deviance"]] <- deviance
    penreg[["x_resid"]] <- x_resid
    penreg[["z_resid"]] <- z_resid
    
    # Post selection estimation
    x_select <- x_resid[, as.numeric(penreg$beta) != 0]
    
    
    
    # Checking issues with hdfe - turns out collinearity prevents estimation 
    # because matrix decomposition not possible
    
    # source(paste0(function_dir,"Corrected_hdfeppml_functions.R"))
    
    
    ppml_temp <- hdfeppml_corrected(y = y, x = x_select, 
                                    fes = fes, tol = 1e-08, hdfetol = 1e-04, mu = penreg$mu, 
                                    colcheck_x = TRUE, colcheck_x_fes = TRUE)
    
    single<-row.names(penreg$beta)[which(penreg$beta!=0&!is.na(penreg$beta))]
    
    if(length(single)==1){
      row.names(ppml_temp$coefficients)<-single
    }
    
    check<-merge(penreg$beta,ppml_temp$coefficients,by="row.names")
    
    colnames(check)<-c("Var","Pen","Unpen")
    
    # Get the vars that survived post-selection (collinearity test)
    get<-filter(check,!is.na(Unpen))
    get$Unpen_se=ppml_temp$se
    get$SERatio=round((abs(get$Unpen)/abs(get$Unpen_se)),1)
    get$RatioPenUnpen=round(get$Pen/get$Unpen,2)

    
    penreg$pencoefs <- penreg$beta
    penreg$beta[which(as.logical(penreg$beta != 0)), 1] <- ppml_temp$coefficients
    b[include_x] <- penreg$beta
    mu <- ppml_temp$mu
    bic <- ppml_temp$bic
    deviance <- ppml_temp$deviance
    
    results<-list(Lambda=lambda,Gamma=gamma,Model=penreg,
                  Coefs=check,Nonzero=get,mu=mu,BIC=bic)
    
    out[[i]]<-results

}

save(out,file=paste0(output_dir,paste0(PROD,"_ppml_models_sensitivities.rda")))

# Create and save sensitivity policy coefficients for
# (a) Unpenalised coefficient values
# (b) Doubling and trebling the penalty
first<-out[[1]]$Coefs
first<-select(first,-Pen)
colnames(first)<-c("name","coef")
policy_coefs<-first
save(policy_coefs,file=paste0(output_dir,paste0(PROD,"_policy_coefs_unpen_lambda1.rda")))

second<-out[[2]]$Coefs
second<-select(second,-Unpen)
colnames(second)<-c("name","coef")
policy_coefs<-second
save(policy_coefs,file=paste0(output_dir,paste0(PROD,"_policy_coefs_pen_lambda2.rda")))

third<-out[[2]]$Coefs
third<-select(third,-Pen)
colnames(third)<-c("name","coef")
policy_coefs<-third
save(policy_coefs,file=paste0(output_dir,paste0(PROD,"_policy_coefs_unpen_lambda2.rda")))

fourth<-out[[3]]$Coefs
fourth<-select(fourth,-Unpen)
colnames(fourth)<-c("name","coef")
policy_coefs<-fourth
save(policy_coefs,file=paste0(output_dir,paste0(PROD,"_policy_coefs_pen_lambda3.rda")))

fifth<-out[[3]]$Coefs
fifth<-select(fifth,-Pen)
colnames(fifth)<-c("name","coef")
policy_coefs<-fifth
save(policy_coefs,file=paste0(output_dir,paste0(PROD,"_policy_coefs_unpen_lambda3.rda")))


