#################################################
# Model of trade in Manufactured dairy products #
#################################################

# This file calibrates model baseline(s), for specific products

#library(fixest)
library(tidyverse)
library(data.table)
library(fixest)
library(openxlsx)
library(glmnet)

# Directories

table_fn<-"./E. Output/Tables/"
plot_fn<-"./E. Output/Plots/"
data_dir<-"./A. Data/"
output_dir<-"./E. Output/"
function_dir<-"./F. Functions/"


##############
# PARAMETERS #
##############

# PRODUCT/SECTOR
PROD="CHS"

# REFERENCE COUNTRY
REF="USA"

# WRITE OUTPUT FILES 
# turn off if checking/changing code so as not to overwrite
WRITE=TRUE

#############
# FUNCTIONS #
#############

source(paste0(function_dir,"Baseline summary functions.R"))

# This function extracts the column names from the matrices
substrRight <- function(x, n){
  substr(x, nchar(x)-n+1, nchar(x))}

#This function extracts coefficients from the glmnet results
extract_coef <- function(o, s) { 
  index <- which(coef(o, s) != 0) 
  data.frame(name=rownames(coef(o))[index], coef=coef(o, s)[index]) }

#############
# LOAD DATA #
#############

# All data includes bilateral trade flows that are zero in every year in the panel
df<-readRDS(file=paste0(data_dir,PROD,"_mdl_data.rds"))

# If REF=AUS, leave reference as is, otherwise change to different country
if(REF!="AUS"){
  # (1) RECALCULATE REFERENCE  
  df<-mutate(df, Origin=as.character(Origin),
             Destination=as.character(Destination),
             Origin=ifelse(Origin=="0-AUS","AUS",ifelse(Origin==REF,paste0("0-",REF),Origin)),
             Destination=ifelse(Destination=="0-AUS","AUS",ifelse(Destination==REF,paste0("0-",REF),Destination)))
  # (2) RECALCULATE COUNTRY-YEAR FACTORS
  df<-mutate(df,Origin=factor(Origin),Destination=factor(Destination),Pair=factor(Pair),
             OriginYear=interaction(Origin,factor(Year)),DestinationYear=interaction(Destination,factor(Year)))
}

# Add EU interaction effects
df<-df%>%
  mutate(INV_X_EU = INV_X*EUN,
         INV_X_EU_L1 = INV_X_L1*EUN,
         INV_X_EU_L2 = INV_X_L2*EUN,
         INV_X_EU_L3 = INV_X_L3*EUN,
         DEV_X_EU = DEV_X*EUN,
         DEV_X_EU_L1 = DEV_X_L1*EUN,
         DEV_X_EU_L2 = DEV_X_L2*EUN,
         DEV_X_EU_L3 = DEV_X_L3*EUN,
         DCP_X_EU = DCP_X*EUN,
         DCP_X_EU_L1 = DCP_X_L1*EUN,
         DCP_X_EU_L2 = DCP_X_L2*EUN,
         DCP_X_EU_L3 = DCP_X_L3*EUN,
         DRY_X_EU = DRY_X*EUN,
         DRY_X_EU_L1 = DRY_X_L1*EUN,
         DRY_X_EU_L2 = DRY_X_L2*EUN,
         DRY_X_EU_L3 = DRY_X_L3*EUN,
         LIV_X_EU = LIV_X*EUN,
         LIV_X_EU_L1 = LIV_X_L1*EUN,
         LIV_X_EU_L2 = LIV_X_L2*EUN,
         LIV_X_EU_L3 = LIV_X_L3*EUN,
         CRP_X_EU = CRP_X*EUN,
         CRP_X_EU_L1 = CRP_X_L1*EUN,
         CRP_X_EU_L2 = CRP_X_L2*EUN,
         CRP_X_EU_L3 = CRP_X_L3*EUN,
         OTH_X_EU = OTH_X*EUN,
         OTH_X_EU_L1 = OTH_X_L1*EUN,
         OTH_X_EU_L2 = OTH_X_L2*EUN,
         OTH_X_EU_L3 = OTH_X_L3*EUN)

# Filter the data so that data is not missing in the year that we estimate the simulation model
# First, drop any rows for which there is no trade (in sum) for the simulation year
zero_trade<-filter(df,Year==max(df$Year),DestinationYearTrade==0|OriginYearTrade==0)
drop_isos<-unique(zero_trade$Origin)
df<-filter(df,!Destination%in%all_of(drop_isos),!Origin%in%all_of(drop_isos))
df<-droplevels(df)
# Drop any countries for which domestic trade is zero 
zero_domestic<-filter(df,External==0&TradeMillions==0,Year==max(df$Year))
zero_domestic_isos<-unique(zero_domestic$Origin)
df<-filter(df,!Destination%in%all_of(zero_domestic_isos),!Origin%in%all_of(zero_domestic_isos))
df<-droplevels(df)


###############################
# FIT SELECTED BASELINE MODEL #
###############################

# Calculate policy offset
load(file=paste0(output_dir,paste0(PROD,"_policy_coefs.rda")))

# Get policy effects
effects<-get_policy_effects_glmnet(Xall=df,coefs=policy_coefs,
                                   DS_vars="DRY|LIV|CRP|DAR|DCP|DEV|INV|OTH|DAR|TAR|TBR|TGR")

# Add to data
df<-merge(df,dplyr::select(effects,Origin,Destination,Year,ln_tij_Policies),all.x=TRUE)


# Construct sparse matrix to fit fixed effects 
df_temp<-df%>%
  filter(DirectionTrade>0)%>%
  mutate(OriginYear=factor(as.character(OriginYear)),DestinationYear=factor(as.character(DestinationYear)),
         Direction=factor(Direction))%>%
  ungroup()

# (a) Origin.Year 
OriginYear.mat<-model.matrix(~OriginYear,data=df_temp)[,-1]
OriginYear.names<-colnames(OriginYear.mat)
OriginYear.names<-substrRight(OriginYear.names,8)
colnames(OriginYear.mat)<-paste0(OriginYear.names,".O")
# Keep a a record of the number of export:year dummies
n_orig<-length(OriginYear.names)
# (b) Destination.Year
DestinationYear.mat<-model.matrix(~DestinationYear,data=df_temp)[,-1]
DestinationYear.names<-colnames(DestinationYear.mat)
DestinationYear.names<-substrRight(DestinationYear.names,8)
colnames(DestinationYear.mat)<-paste0(DestinationYear.names,".D")
# Keep a a record of the number of import:year dummies
n_dest<-length(DestinationYear.names)
# (c) Country pairs (2858 no intercept)
Direction.mat<-model.matrix(~Direction,data=df_temp)[,-1]
Direction.names<-colnames(Direction.mat) 
Direction.names<-substrRight(Direction.names,7)
colnames(Direction.mat)<-Direction.names
# (d) Sparse matrix for estimation
X = cbind(OriginYear.mat,DestinationYear.mat,Direction.mat)
# Remove unneeded matrices
rm(OriginYear.mat,DestinationYear.mat,Direction.mat)
# Create sparse matrix of predictive variables
X_sparse<-Matrix(X, sparse = TRUE) 
gc()
# (e) y
y=df_temp$TradeMillions
# (f) Offset
fe_offset=df_temp$ln_tij_Policies
# (g) Estimate
bln_mdl = glmnet(X_sparse,y,alpha=1.0,lambda=0,family=quasipoisson(link="log"),
                   intercept=FALSE,offset=fe_offset)
# (c) Coeffs
fe_betas<-coef(bln_mdl)[-1]
# (d) Extract coefs
fe_coefs<-extract_coef(bln_mdl,s=0)

if(WRITE==TRUE){
  write.csv(fe_coefs,paste0(output_dir,PROD,"_baseline_FE_initial.csv"))
  saveRDS(bln_mdl,file=paste0(output_dir,PROD,"_PPML_BASELINE.rds"))
}

###################################################
# ADD COST AND COEFFICIENT ESTIMATES TO DATAFRAME #
###################################################

# Get coefficients with names
bln_coefs<-policy_coefs
colnames(bln_coefs)<-c("Variable","Coef")

fe<-fe_coefs
gamma_ij<-fe[!grepl("\\.O|\\.D",fe$name),]
colnames(gamma_ij)<-c("Direction","gamma_ij")
hist(gamma_ij$gamma_ij)

# Get Exporter-Year effects
eta_it<-fe[grepl("\\.O",fe$name),]
colnames(eta_it)<-c("OriginYear","eta_it")
hist(eta_it$eta_it)
eta_it$OriginYear<-gsub("\\.O","",eta_it$OriginYear)
eta_it$OriginYear<-gsub(REF,paste0("0-",REF),eta_it$OriginYear)

# Get Importer-Year effects
psi_jt<-fe[grepl("\\.D",fe$name),]
colnames(psi_jt)<-c("DestinationYear","psi_jt")
hist(psi_jt$psi_jt)
psi_jt$DestinationYear<-gsub("\\.D","",psi_jt$DestinationYear)
psi_jt$DestinationYear<-gsub(REF,paste0("0-",REF),psi_jt$DestinationYear)

# Add directional fixed effects to mdl_data
bln_data<-merge(x=df,y=gamma_ij,by=c("Direction"),all.x=TRUE)

# Add Exporter-Year fixed effects to bln_data
bln_data<-merge(x=bln_data,y=eta_it,by=c("OriginYear"),all.x=TRUE)

# Add Importer-Year fixed effects to bln_data
bln_data<-merge(x=bln_data,y=psi_jt,by=c("DestinationYear"),all.x=TRUE)

# Replace reference values with 0
bln_data<-bln_data%>%
  mutate(gamma_ij=ifelse(is.na(gamma_ij)&Direction=="0-INTRA",0,gamma_ij),
         eta_it=ifelse(is.na(eta_it)&OriginYear==paste0("0-",REF,".2013"),0,eta_it),
         psi_jt=ifelse(is.na(psi_jt)&DestinationYear==paste0("0-",REF,".2013"),0,psi_jt))

# Save copuy of chosen policy effects for later reference
policy_effects<-effects
if(WRITE==TRUE){
  write.csv(policy_effects,paste0(output_dir,PROD,"_POLICY_EFFECTS.csv"))
}

# Add the policy variables to the baseline data
bln_data<-merge(bln_data,dplyr::select(policy_effects,-ln_tij_Policies),by=c("Origin","Destination","Year"),all.x=TRUE)

# Fit model baseline
bln_mdl = glmnet(X_sparse,y,alpha=1.0,lambda=0,family=quasipoisson(link="log"),
                 intercept=FALSE,offset=fe_offset)

# Add predicted trade
bln_data<-bln_data%>%
  mutate(Xij_hat  = exp(gamma_ij+ln_tij_Policies+eta_it+psi_jt),
         eps=TradeMillions-Xij_hat,
         eps_ratio=ifelse(is.na(Xij_hat),NA,ifelse(TradeMillions==0,0.001/Xij_hat,TradeMillions/Xij_hat)),
         ln_eps_ratio=ifelse(eps_ratio==0,NA,log(eps_ratio)))

ggplot(filter(bln_data,Origin!=Destination),aes(x=TradeMillions,y=Xij_hat))+geom_point()+theme_minimal()

# Add fitted trade costs to model baseline data
bln_data<-mutate(bln_data,
                 tij_bar=exp(gamma_ij+ln_eps_ratio), # Fixed/time invariant directional costs
                 tij_bln=exp(gamma_ij+ln_tij_Policies+ln_eps_ratio)) # Total trade costs


#########################################
# PREDICT BLIATERAL COSTS/FIXED EFFECTS #
#########################################

# Add data on gravity variables to the existing data on trade costs
cost_mdl_data<-filter(bln_data,Year==max(bln_data$Year),Origin!=Destination)

# Model for estimating/predicting directional trade costs (ex policies i.e. tij_bar) 
# NOTE THAT IN some estimations the presence of COMLANG causes convergence issues so may need to be excluded
cost_mdl<-fepois(
  tij_bar ~ LN_DISTW + CONTIG + COMLANG + COLNY| Origin + Destination,
  data = cost_mdl_data,glm.iter=500)

cost_mdl_results<-etable(cost_mdl)

# Calculate tij
cost_mdl_data<-cost_mdl_data%>%
  mutate(tij_x_policies = predict(cost_mdl, cost_mdl_data)) %>%
  dplyr::select(Origin, Destination, tij_x_policies)

# Baseline data, for most recent year
bln<-bln_data%>%
  filter(Year == max(bln_data$Year))%>%
  left_join(dplyr::select(cost_mdl_data,Origin,Destination,tij_x_policies), by = c("Origin", "Destination"))%>%
  mutate(tij_na = ifelse(is.na(tij_bar),1,0),
    tij_bar = ifelse(is.na(tij_bar), tij_x_policies, tij_bar),
    tij_bln = ifelse(is.na(tij_bln), exp(log(tij_bar)+ln_tij_Policies), tij_bln)) %>%
  dplyr::select(-tij_x_policies)

bln_na<-dplyr::select(filter(bln,tij_na==1),Origin,Destination,tij_bar,tij_bln,eta_it,psi_jt)

# Predict trade using gravity identity and add epsilon as unobserved trade costs for the zero trade areas
bln<-bln%>%
  mutate(Xf=exp(log(tij_bln)+eta_it+psi_jt),
         eps_zero=TradeMillions-Xf,
         eps_ratio_zero=ifelse(is.na(Xf),NA,ifelse(TradeMillions==0,0.0000001/Xf,TradeMillions/Xf)),
         ln_eps_ratio_zero=ifelse(eps_ratio_zero==0,NA,log(eps_ratio_zero)),
         tij_bar=ifelse(tij_na==1,exp(log(tij_bar)+ln_eps_ratio_zero),tij_bar),
         tij_bln=ifelse(tij_na==1,exp(log(tij_bln)+ln_eps_ratio_zero),tij_bln))

bln_na<-dplyr::select(filter(bln,tij_na==1),Origin,Destination,tij_bar,tij_bln,eta_it,psi_jt,ln_eps_ratio_zero,Xf)

# Check the calculations - apply the new trade costs (tij_bln) to see what the fitted trade looks like
chk<-bln%>%
  mutate(Xf=exp(log(tij_bln)+eta_it+psi_jt),
         eps=TradeMillions-Xf,
         eps_ratio=TradeMillions/Xf)%>%
  dplyr::select(Origin,Destination,External,TradeMillions,Xf,eps,eps_ratio)

ggplot(chk,aes(x=TradeMillions,y=Xf))+geom_point()+theme_minimal()

hist(log(bln$tij_bln))

# Fit model to readjust the fixed effects

# (a) Origin 
Origin.mat<-model.matrix(~Origin,data=bln,contrasts.arg = list(Origin = contrasts(bln$Origin, contrasts = FALSE)))[,-1]
Origin.names<-colnames(Origin.mat)
Origin.names<-substrRight(Origin.names,3)
colnames(Origin.mat)<-paste0(Origin.names,".O")
# (b) Destination.Year
Destination.mat<-model.matrix(~Destination,data=bln)[,-1]
Destination.names<-colnames(Destination.mat)
Destination.names<-substrRight(Destination.names,3)
colnames(Destination.mat)<-paste0(Destination.names,".D")
# (c) Sparse matrix for estimation
X = cbind(Origin.mat,Destination.mat)
# Remove unneeded matrices
rm(Origin.mat,Destination.mat,Direction.mat)
# Create sparse matrix of predictive variables
X_sparse<-Matrix(X, sparse = TRUE) 
gc()
# (d) y
y=bln$TradeMillions
# (e) Offset
fe_offset=log(bln$tij_bln)
# (f) Estimate
fit_fe_bln = glmnet(X_sparse,y,alpha=1.0,lambda=0,family=quasipoisson(link="log"),
                 intercept=FALSE,offset=fe_offset)
# (g) Extract coefs
fe_coefs<-extract_coef(fit_fe_bln,s=0)

if(WRITE==TRUE){
write.csv(fe_coefs,paste0(output_dir,PROD,"_baseline_FE_final.csv"))
}

fe<-fe_coefs

# Get Exporter effects
eta_i<-fe[grepl("\\.O",fe$name),]
colnames(eta_i)<-c("Origin","eta_i")
hist(eta_i$eta_i)
eta_i$Origin<-gsub("\\.O","",eta_i$Origin)
eta_i$Origin<-gsub(REF,paste0("0-",REF),eta_i$Origin)

# Get Importer-Year effects
psi_j<-fe[grepl("\\.D",fe$name),]
colnames(psi_j)<-c("Destination","psi_j")
hist(psi_j$psi_j)
psi_j$Destination<-gsub("\\.D","",psi_j$Destination)
psi_j$Destination<-gsub(REF,paste0("0-",REF),psi_j$Destination)

# Add Exporter-Year fixed effects to bln_data
bln<-merge(x=bln,y=eta_i,by=c("Origin"),all.x=TRUE)

# Add Importer-Year fixed effects to bln_data
bln<-merge(x=bln,y=psi_j,by=c("Destination"),all.x=TRUE)

# # Replace reference value(s) with 0
bln<-bln%>%
   mutate(psi_j=ifelse(is.na(psi_j)&Destination==paste0("0-",REF),0,psi_j))

# Variables, include baseline (time invariant) origin and destination fixed effects 
bln<-bln%>%
  mutate(Xij_bln = exp(log(tij_bln)+eta_i+psi_j))%>% # Directional trade flows
  group_by(Origin)%>%
  mutate(Xi_bln = sum(Xij_bln*(Direction!="0-INTRA")))%>% # Total exports by origin
  ungroup()%>%
  group_by(Destination)%>%
  mutate(Mi_bln = sum(Xij_bln*(Direction!="0-INTRA")))%>% # Total imports by origin
  ungroup()

# Baseline aggregates
bln<-bln%>%
  group_by(Origin,Year)%>%
  mutate(Yi=sum(Xij_bln))%>%
  ungroup()%>%
  group_by(Destination,Year)%>%
  mutate(Ej=sum(Xij_bln))%>%
  ungroup()%>%
  group_by(Year)%>%
  mutate(Er=max(ifelse(Destination==paste0("0-",REF),Ej,NA),na.rm=TRUE))%>%
  ungroup()

# Outward multilateral resistance (Pi_i) and Inward Multilateral resistance (Rho_j) indices
bln<-bln%>%
  group_by(Year)%>%
  mutate(Pi_i=Yi*Er/exp(eta_i),Rho_j=Ej/(exp(psi_j)*Er))%>%
  ungroup()

# Save the baseline data
if(WRITE==TRUE){
saveRDS(bln,file=paste0(output_dir,PROD,"_BASELINE_ESTIBRATED_AGGR.rds"))
}












