#########################################################
# SIMULATED EFFECTS OF POLICY CHANGE ON TRADE IN CHEESE #
#########################################################

# Focus is on the components of domestic support in the EU

library(tidyverse)
library(data.table)
library(fixest)
library(openxlsx)
library(glmnet)

# Directories


table_fn<-"./E. Output/Tables/"
plot_fn<-"./E. Output/Plots/"
data_dir<-"./A. Data/"
output_dir<-"./E. Output/"
function_dir<-"./F. Functions/"

#############
# SCENARIOS #
#############

scenario_vars<-list(EU_DRY=c("DRY"),EU_LIV=c("LIV"),EU_CRP=c("CRP"),EU_OTH=c("OTH"),EU_CPL=c("DRY","CRP","LIV","OTH"),
                    EU_INC=c("DCP"),EU_INV=c("INV"),EU_DEV=c("DEV"),EU_DCP=c("DCP","INV","DEV"),
                    EU_ALL=c("DRY","LIV","CRP","OTH","DCP","INV","DEV"),
                    ALL_CPL=c("DRY","LIV","CRP","OTH","DAR","TAR","TBR"),
                    ALL_DCP=c("DCP","INV","DEV","TGR"),
                    ALL=c("DRY","LIV","CRP","OTH","DCP","INV","DEV","DAR","TAR","TBR","TGR"))

scenario_select<-list(EU_DRY=c("DRY"),EU_LIV=c("LIV"),EU_CRP=c("CRP"),EU_OTH=c("OTH"),EU_CPL=c("DRY|LIV|CRP|OTH"),
                      EU_INC=c("DCP"),EU_INV=c("INV"),EU_DEV=c("DEV"),EU_DCP=c("DCP|INV|DEV"),
                      EU_ALL=c("DRY|LIV|CRP|OTH|DCP|INV|DEV"),
                      ALL_CPL=c("DRY|LIV|CRP|OTH|DAR|TAR|TBR"),
                      ALL_DCP=c("DCP|INV|DEV|TGR"),
                      ALL=c("DRY|LIV|CRP|OTH|DCP|INV|DEV|DAR|TAR|TBR|TGR"))

scenarios<-names(scenario_vars)


##############
# PARAMETERS #
##############

# PRODUCT/SECTOR
PROD="CHS"

# REFERENCE COUNTRY
REF="USA"

# MAXIMUM LAGS IN MODEL
MAX_LAG="L3"

# SIZE OF SHOCK
SIZE=-0.1

# TRADE COST ELASTICITY
sigma_ij<-6.4

# Countries
shock_isos<-"ALL"

# SCENARIO
SCENARIO="EU_ALL"

# Convergence
crit<-0.001

#############
# FUNCTIONS #
#############

source(paste0(function_dir,"Policy shock functions.R"))

# This function extracts the column names from the matrices
substrRight <- function(x, n){
  substr(x, nchar(x)-n+1, nchar(x))}

#This function extracts coefficients from the glmnet results
extract_coef <- function(o, s) { 
  index <- which(coef(o, s) != 0) 
  data.frame(name=rownames(coef(o))[index], coef=coef(o, s)[index]) }

#################
# LOAD BASELINE #
#################

# Calibrated baseline dataframe
bln<-readRDS(file=paste0(output_dir,PROD,"_BASELINE_ESTIBRATED_AGGR.rds"))
# Baseline model - for coefficients
bln_mdl<-readRDS(file=paste0(output_dir,PROD,"_PPML_BASELINE.rds"))
# Original baseline data
df<-readRDS(file=paste0(data_dir,PROD,"_mdl_data.rds"))

# Add EU interaction effects
df<-df%>%
  mutate(INV_X_EU = INV_X*EUN,
         INV_X_EU_L1 = INV_X_L1*EUN,
         INV_X_EU_L2 = INV_X_L2*EUN,
         INV_X_EU_L3 = INV_X_L3*EUN,
         DEV_X_EU = DEV_X*EUN,
         DEV_X_EU_L1 = DEV_X_L1*EUN,
         DEV_X_EU_L2 = DEV_X_L2*EUN,
         DEV_X_EU_L3 = DEV_X_L3*EUN,
         DCP_X_EU = DCP_X*EUN,
         DCP_X_EU_L1 = DCP_X_L1*EUN,
         DCP_X_EU_L2 = DCP_X_L2*EUN,
         DCP_X_EU_L3 = DCP_X_L3*EUN,
         DRY_X_EU = DRY_X*EUN,
         DRY_X_EU_L1 = DRY_X_L1*EUN,
         DRY_X_EU_L2 = DRY_X_L2*EUN,
         DRY_X_EU_L3 = DRY_X_L3*EUN,
         LIV_X_EU = LIV_X*EUN,
         LIV_X_EU_L1 = LIV_X_L1*EUN,
         LIV_X_EU_L2 = LIV_X_L2*EUN,
         LIV_X_EU_L3 = LIV_X_L3*EUN,
         CRP_X_EU = CRP_X*EUN,
         CRP_X_EU_L1 = CRP_X_L1*EUN,
         CRP_X_EU_L2 = CRP_X_L2*EUN,
         CRP_X_EU_L3 = CRP_X_L3*EUN,
         OTH_X_EU = OTH_X*EUN,
         OTH_X_EU_L1 = OTH_X_L1*EUN,
         OTH_X_EU_L2 = OTH_X_L2*EUN,
         OTH_X_EU_L3 = OTH_X_L3*EUN)

# Filter the data so that data is not missing in the year that we estimate the simulation model
# First, drop any rows for which there is no trade (in sum) for the simulation year
zero_trade<-filter(df,Year==max(df$Year),DestinationYearTrade==0|OriginYearTrade==0)
drop_isos<-unique(zero_trade$Origin)
df<-filter(df,!Destination%in%all_of(drop_isos),!Origin%in%all_of(drop_isos))
df<-droplevels(df)
# Drop any countries for which domestic trade is zero 
zero_domestic<-filter(df,External==0&TradeMillions==0,Year==max(df$Year))
zero_domestic_isos<-unique(zero_domestic$Origin)
df<-filter(df,!Destination%in%all_of(zero_domestic_isos),!Origin%in%all_of(zero_domestic_isos))
df<-droplevels(df)


# Drop ISL?
# df<-filter(df,Origin!="ISL",Destination!="ISL")

################
# POLICY SHOCK #
################

# Get coefficients with names
load(paste0(output_dir,PROD,"_policy_coefs.rda"))
bln_coefs<-policy_coefs
colnames(bln_coefs)<-c("Variable","Coef")


# Get vartiables to shock
get_shock_vars_m<-bln_coefs[grepl("_M",bln_coefs$Variable)&grepl(scenario_select[[SCENARIO]],bln_coefs$Variable),]$Variable
get_shock_vars_x<-bln_coefs[grepl("_X",bln_coefs$Variable)&grepl(scenario_select[[SCENARIO]],bln_coefs$Variable),]$Variable

  
policy_shock<-get_policy_shocks_glmnet(coefs=policy_coefs,mdl_data=bln,
                                shock_vars_m = get_shock_vars_m,
                                shock_vars_x = get_shock_vars_x, 
                                shock_val = SIZE,
                                shock_country = shock_isos,
                                DS_vars="DRY|LIV|CRP|OTH|DCP|INV|DEV|DAR|TAR|TBR|TGR",
                                ref=REF)

# Create counterfactual trade costs variable
policy_shock<-mutate(policy_shock,ln_tij_Policies_cfl=ln_tij_Policies)

# Calculate the size of the changes to domestic support - in dollar values
# Get the value of variables affected
ds_shocked<-c(get_shock_vars_m,get_shock_vars_x)
# Get the data
ds_shocked_nom_vars<-sub("R_","N_",ds_shocked)

##################
# COUNTERFACTUAL #
##################

# Add policy shock to data
ge_data<-merge(bln,dplyr::select(policy_shock,Origin,Destination,Year,ln_tij_Policies_cfl),by=c("Origin","Destination","Year"),all.x=TRUE)

# Recalculate trade costs
ge_data<-mutate(ge_data,tij_cfl=exp(log(tij_bar)+ln_tij_Policies_cfl))

# Re-estimate/solve importer and exporter fixed effects
# (a) Origin 
Origin.mat<-model.matrix(~Origin,data=ge_data,contrasts.arg = list(Origin = contrasts(ge_data$Origin, contrasts = FALSE)))[,-1]
Origin.names<-colnames(Origin.mat)
Origin.names<-substrRight(Origin.names,3)
colnames(Origin.mat)<-paste0(Origin.names,".O")
# (b) Destination.Year
Destination.mat<-model.matrix(~Destination,data=ge_data)[,-1]
Destination.names<-colnames(Destination.mat)
Destination.names<-substrRight(Destination.names,3)
colnames(Destination.mat)<-paste0(Destination.names,".D")
# (c) Sparse matrix for estimation
X = cbind(Origin.mat,Destination.mat)
# Remove unneeded matrices
rm(Origin.mat,Destination.mat)
# Create sparse matrix of predictive variables
X_sparse<-Matrix(X, sparse = TRUE) 
gc()
# (d) y
y=ge_data$Xij_bln
# (e) Offset
fe_offset=log(ge_data$tij_cfl)
# (f) Estimate
glmnet.control(mxitnr = 100) # Non-convergence occurs without higher number of IRLS iterations 
fit_fe_cfl = glmnet(X_sparse,y,alpha=1.0,lambda=0,family=quasipoisson(link="log"),
                    intercept=FALSE,offset=fe_offset,standardize=FALSE)
# (g) Extract coefs
fe_coefs<-extract_coef(fit_fe_cfl,s=0)
fe<-fe_coefs

# Get Exporter effects
cfl_eta_i<-fe[grepl("\\.O",fe$name),]
colnames(cfl_eta_i)<-c("Origin","cfl_eta_i")
hist(cfl_eta_i$cfl_eta_i)
cfl_eta_i$Origin<-gsub("\\.O","",cfl_eta_i$Origin)
cfl_eta_i$Origin<-gsub(REF,paste0("0-",REF),cfl_eta_i$Origin)

# Get Importer effects
cfl_psi_j<-fe[grepl("\\.D",fe$name),]
colnames(cfl_psi_j)<-c("Destination","cfl_psi_j")
hist(cfl_psi_j$cfl_psi_j)
cfl_psi_j$Destination<-gsub("\\.D","",cfl_psi_j$Destination)
cfl_psi_j$Destination<-gsub(REF,paste0("0-",REF),cfl_psi_j$Destination)

# Replace reference values with 0
ge_data<-merge(x=ge_data,y=cfl_eta_i,by=c("Origin"),all.x=TRUE)
ge_data<-merge(x=ge_data,y=cfl_psi_j,by=c("Destination"),all.x=TRUE)

ge_data<-ge_data%>%
  mutate(cfl_psi_j=ifelse(is.na(cfl_psi_j)&Destination==paste0("0-",REF),0,cfl_psi_j))

# Counterfactual outward multilateral resistance (Pi_i) and Inward Multilateral resistance (Rho_i) indices
ge_data<-ge_data%>%
  mutate(Pi_i_cfl=Yi*Er/exp(cfl_eta_i),Rho_j_cfl=Ej/(exp(cfl_psi_j)*Er))

# Fitted trade values from counterfactual fit
ge_data<-ge_data%>%
  mutate(Xij_cfl = exp(log(tij_cfl)+cfl_eta_i+cfl_psi_j)) %>%
  group_by(Origin)%>%
  mutate(Xi_cfl = sum(Xij_cfl * (Pair != "0-INTRA"))) %>%
  ungroup()%>%
  group_by(Destination) %>%
  mutate(Mi_cfl = sum(Xij_cfl * (Pair != "0-INTRA"))) %>%
  ungroup()


#############
# GE set-up #
#############

# Full endowment general equilibrium effects from the removal of policy change 
# Four-stage iterative procedure that allows for endogenous factory-gate prices, income, expenditure and trade to
# adjust to the counterfactual shock.

# Calculate changes in trade costs d_tij, under the counterfactual, and the exogenous trade deficit/surplus (phi)
ge_data <- ge_data%>%
  mutate(d_tij = tij_cfl / tij_bln,
         phi = ifelse(Origin == Destination, Ej / Yi, 0))%>%
  group_by(Origin)%>%
  mutate(phi = max(phi)) %>%
  ungroup()

# Calculate changes in prices for exports (d_p_i) and imports (d_p_j) [Based on market clearing and trade cost elasticity]
ge_data<-ge_data%>%
  group_by(Origin)%>%
  mutate(d_p_i = ((exp(cfl_eta_i) / Er) / (exp(eta_i) /Er))^(1 /(1 - sigma_ij))) %>%
  ungroup()%>%
  group_by(Destination) %>%
  mutate(d_p_j = ifelse(Origin == Destination, d_p_i, 0),
         d_p_j = max(d_p_j))%>%
  ungroup()

#######################
# CONDITIONAL INDICES #
#######################
Xcon<-ge_data%>%
  mutate(Origin=as.character(Origin))%>%
  group_by(Origin)%>%
  summarise(Yi_bln=round(sum(Xij_bln),0),
            Xi_bln=round(sum(Xij_bln * (Pair != "0-INTRA")),0),
            Xi_cfl=round(sum(Xij_cfl * (Pair != "0-INTRA")),0),
            Xii_bln=round(sum(Xij_bln * (Pair == "0-INTRA")),0),
            Xii_cfl=round(sum(Xij_cfl * (Pair == "0-INTRA")),0),
            Pi_i_bln=max(Pi_i),
            Pi_i_cfl=max(Pi_i_cfl))%>%
  mutate(dXi=round((Xi_cfl/Xi_bln-1)*100,2),
         dXii=round((Xii_cfl/Xii_bln-1)*100,2),
         dPi=round(((Pi_i_cfl/Pi_i_bln)^(1/(1-sigma_ij))- 1)*100,2))

Mcon<-ge_data%>%
  mutate(Destination=as.character(Destination))%>%
  group_by(Destination)%>%
  summarise(Ej_bln=round(sum(Xij_bln),0),
            Mi_bln=round(sum(Xij_bln * (Pair != "0-INTRA")),0),
            Mi_cfl=round(sum(Xij_cfl * (Pair != "0-INTRA")),0),
            Rho_j_bln=max(Rho_j),
            Rho_j_cfl=max(Rho_j_cfl))%>%
  mutate(dMi=round((Mi_cfl/Mi_bln-1)*100,2),
         dRho=round(((Rho_j_cfl/Rho_j_bln)^(1/(1-sigma_ij))- 1)*100,2))

ConditionalEffects<-merge(x=Xcon,y=Mcon,by.x=c("Origin"),by.y=c("Destination"),all.x=TRUE)%>%
  mutate(dW_con=round((((Xii_cfl/Ej_bln)/(Xii_bln/Ej_bln))^(1/(1-sigma_ij))-1)*100,2))%>%
  dplyr::select(-Pi_i_bln,-Pi_i_cfl,-Rho_j_bln,-Rho_j_cfl)

col_order<-c("Origin","Yi_bln","Ej_bln","Xi_bln","Mi_bln","Xii_bln","Xi_cfl","dXi","Mi_cfl","dMi","Xii_cfl","dXii","dPi","dRho","dW_con")

ConditionalEffects<-ConditionalEffects[,col_order]

colnames(ConditionalEffects)<-c("ISO code","Production","Expenditure","Exports","Imports","Domestic trade",
                 "Counterfactual exports","Change in exports (%)","Counterfactual imports","Change in imports (%)",
                 "Counterfactual domestic trade","Change in domestic trade (%)",
                 "Change in outward mulilateral resistance (%)","Change in inward mulilateral resistance (%)",
                 "Change in consumer welfare (%)")

#################
# GE SIMULATION #
#################

# Construct reference variables
ge_data <- ge_data %>%
  mutate(
    Pi_i_cfl_0 = Pi_i_cfl, # Outward multilateral resistance
    Rho_j_cfl_0 = Rho_j_cfl, # Inward multilateral resistance
    d_Pi_i_full_0 = 1, 
    d_Rho_j_full_0 = 1,
    d_p_i_0 = d_p_i, # Change in export prices
    d_p_j_0 = d_p_j, # Change in import prices
    cfl_eta_i_0 = cfl_eta_i,  # Counterfactual exporter fixed effects
    cfl_psi_j_0 = cfl_psi_j, # Counterfactual importer fixed effects
    Xij_hat_0 = Xij_cfl, # Fitted trade from constrained counterfactual model
    Er_cfl_0 = Er # Reference country expenditure
  )

max_dif<-1 # Maximum price difference in data
sd_dif<-1 # Std deviation of price difference in data
change_price_i_old<-0 # Prior maximum price change
max_iter<-20

sd_tol=crit
max_tol=crit

i=1

start.time=Sys.time()

while(i<max_iter&(sd_dif>sd_tol|max_dif>max_tol)){
  
  # Calculate trade flows
  ge_data <- ge_data%>%
    mutate(Xij_1 = Xij_hat_0 * d_p_i_0 * d_p_j_0 / (d_Pi_i_full_0 * d_Rho_j_full_0))
  
  # Refit constrained counterfactual model, with modified trade flows
  
  #########
  # Re-estimate/solve importer and exporter fixed effects
  # (a) Origin 
  Origin.mat<-model.matrix(~Origin,data=ge_data,contrasts.arg = list(Origin = contrasts(ge_data$Origin, contrasts = FALSE)))[,-1]
  Origin.names<-colnames(Origin.mat)
  Origin.names<-substrRight(Origin.names,3)
  colnames(Origin.mat)<-paste0(Origin.names,".O")
  # (b) Destination.Year
  Destination.mat<-model.matrix(~Destination,data=ge_data)[,-1]
  Destination.names<-colnames(Destination.mat)
  Destination.names<-substrRight(Destination.names,3)
  colnames(Destination.mat)<-paste0(Destination.names,".D")
  # (c) Sparse matrix for estimation
  X = cbind(Origin.mat,Destination.mat)
  # Remove unneeded matrices
  rm(Origin.mat,Destination.mat)
  # Create sparse matrix of predictive variables
  X_sparse<-Matrix(X, sparse = TRUE) 
  gc()
  # (d) y
  y=ge_data$Xij_1
  # (e) Offset
  fe_offset=log(ge_data$tij_cfl)
  # (f) Estimate
  glmnet.control(mxitnr = 200) # Non-convergence occurs without higher number of IRLS iterations 
  cfl_trade_mdl_temp = glmnet(X_sparse,y,alpha=1.0,lambda=0,family=quasipoisson(link="log"),
                      intercept=FALSE,offset=fe_offset,standardize=FALSE)
  # (g) Extract coefs
  fe_coefs<-extract_coef(cfl_trade_mdl_temp,s=0)
  fe<-fe_coefs
  
  # Get Exporter effects
  temp_eta_i<-fe[grepl("\\.O",fe$name),]
  colnames(temp_eta_i)<-c("Origin","temp_eta_i")
  hist(temp_eta_i$temp_eta_i)
  temp_eta_i$Origin<-gsub("\\.O","",temp_eta_i$Origin)
  temp_eta_i$Origin<-gsub(REF,paste0("0-",REF),temp_eta_i$Origin)
  
  # Get Importer effects
  temp_psi_j<-fe[grepl("\\.D",fe$name),]
  colnames(temp_psi_j)<-c("Destination","temp_psi_j")
  hist(temp_psi_j$temp_psi_j)
  temp_psi_j$Destination<-gsub("\\.D","",temp_psi_j$Destination)
  temp_psi_j$Destination<-gsub(REF,paste0("0-",REF),temp_psi_j$Destination)
  
  # Replace reference values with 0
  ge_data<-merge(x=ge_data,y=temp_eta_i,by=c("Origin"),all.x=TRUE)
  ge_data<-merge(x=ge_data,y=temp_psi_j,by=c("Destination"),all.x=TRUE)
  
  ge_data<-ge_data%>%
    mutate(temp_psi_j=ifelse(is.na(temp_psi_j)&Destination==paste0("0-",REF),0,temp_psi_j))
  
  # Conditional GE effects
  
  ge_data <- ge_data %>%
    mutate(Xij_hat_1 = exp(log(tij_cfl)+temp_eta_i+temp_psi_j))%>%
    group_by(Origin) %>%
    mutate(Yi_cfl_1 = sum(Xij_hat_1)) %>%
    ungroup() %>%
    mutate(Ej_cfl_1 = ifelse(Pair == "0-INTRA", phi * Yi_cfl_1, 0)) %>%
    group_by(Destination)%>%
    mutate(Ej_cfl_1 = max(Ej_cfl_1)) %>%
    ungroup() %>%
    mutate(Er_cfl_1 = ifelse(Destination == paste0("0-",REF), Ej_cfl_1, 0),
           Er_cfl_1 = max(Er_cfl_1))
  
  # Price changes
  ge_data <- ge_data %>%
    mutate(d_p_i_1 = ((exp(temp_eta_i) / Er_cfl_1) /
                        (exp(cfl_eta_i_0) / Er_cfl_0))^(1 / (1 - sigma_ij)))
  
  ge_data<- ge_data%>%
    group_by(Destination)%>%
    mutate(d_p_j_1 = ifelse(Pair == "0-INTRA", d_p_i_1, 0),
           d_p_j_1 = max(d_p_j_1))%>%
    ungroup()
  
  # Update outward multilateral (Pi_i) and inward multilateral (Rho_j) resistances
  ge_data<-ge_data%>%
    mutate(Pi_i_cfl_1 = (Yi_cfl_1 * Er_cfl_1) / exp(temp_eta_i),
           Rho_j_cfl_1 = Ej_cfl_1 / (exp(temp_psi_j) * Er_cfl_1))
  
  # Update differences in prices
  max_dif <- abs(max(ge_data$d_p_i_0 - change_price_i_old))
  sd_dif <- sd(ge_data$d_p_i_0 - change_price_i_old)
  change_price_i_old <- ge_data$d_p_i_0
  
  # Update variables being tracked
  ge_data <- ge_data %>%
    mutate(
      d_Pi_i_full_1 = Pi_i_cfl_1 / Pi_i_cfl_0,
      d_Rho_j_full_1 = Rho_j_cfl_1 / Rho_j_cfl_0,
      Pi_i_cfl_0 = Pi_i_cfl_1,
      Rho_j_cfl_0 = Rho_j_cfl_1,
      d_Pi_i_full_0 = d_Pi_i_full_1,
      d_Rho_j_full_0 = d_Rho_j_full_1,
      d_p_i_0 = d_p_i_1,
      d_p_j_0 = d_p_j_1,
      cfl_eta_i_0 = temp_eta_i,
      cfl_psi_j_0 = temp_psi_j,
      Xij_hat_0 = Xij_hat_1,
      Er_cfl_0 = Er_cfl_1) %>%
    dplyr::select(-temp_eta_i, -temp_psi_j)
  
  i <- i + 1
  print(i)
  print(max_dif)
  print(sd_dif)
  
}

end.time=Sys.time()
end.time-start.time  


###################
# Final GE values #
###################

ge_data <- ge_data%>%
  mutate(d_p_i_full = ((exp(cfl_eta_i_0) / Er_cfl_0) /
                         (exp(eta_i) / Er))^(1 / (1 - sigma_ij)),
         d_p_j_full = d_p_i_full * (Origin==Destination))%>%
  group_by(Destination) %>%
  mutate(d_p_j_full = max(d_p_j_full))%>%
  ungroup() %>%
  mutate(Yi_full = d_p_i_full * Yi)

ge_data <- ge_data %>%
  mutate(Ej_full = d_p_j_full * Ej * (Pair=="0-INTRA")) %>%
  group_by(Destination) %>%
  mutate(Ej_full = max(Ej_full, na.rm = TRUE)) %>%
  ungroup() %>%
  mutate(Er_full = Ej_full * (Destination == paste0("0-",REF)),
         Er_full = max(Er_full)
  )

ge_data <- ge_data %>%
  mutate(Pi_i_full = Yi_full * Er_cfl_0 / exp(cfl_eta_i_0),
         Rho_j_full = Ej_cfl_1 / (exp(cfl_psi_j_0) * Er_cfl_0))

ge_data <- ge_data %>%
  mutate(Xij_full = (Yi_full * Ej_full * tij_cfl) / (Pi_i_full * Rho_j_full)) %>%
  group_by(Origin) %>%
  mutate(Xi_full = sum(Xij_full * (Pair!="0-INTRA"))) %>%
  ungroup()

# Write GE data
write.csv(ge_data,paste0(output_dir,PROD,"_",SCENARIO,"_GE_OUTPUT_AGGR.csv"))
saveRDS(ge_data,file=paste0(output_dir,PROD,"_",SCENARIO,"_GE_OUTPUT_AGGR.rds"))

# Output table

Xfull<-ge_data%>%
  mutate(Origin=as.character(Origin))%>%
  group_by(Origin)%>%
  summarise(Yi_bln=round(sum(Xij_bln),0),
            Yi_full=round(sum(Xij_full),0),
            Xi_bln=round(sum(Xij_bln * (Pair != "0-INTRA")),0),
            Xi_cfl=round(sum(Xij_cfl * (Pair != "0-INTRA")),0),
            Xi_full=round(sum(Xij_full * (Pair != "0-INTRA")),0),
            Xii_bln=round(sum(Xij_bln * (Pair == "0-INTRA")),0),
            Xii_cfl=round(sum(Xij_cfl * (Pair == "0-INTRA")),0),
            Xii_full=round(sum(Xij_full * (Pair == "0-INTRA")),0),
            Pi_i_bln=max(Pi_i),
            Pi_i_cfl=max(Pi_i_cfl),
            Pi_i_full=max(Pi_i_full),
            d_p_i_cfl=round((max(d_p_i)-1)*100,2),
            d_p_i_full=round((max(d_p_i_full)-1)*100,2))%>%
  mutate(dYi_full=round((Yi_full/Yi_bln-1)*100,2),
         dXi_con=round((Xi_cfl/Xi_bln-1)*100,2),
         dXi_full=round((Xi_full/Xi_bln-1)*100,2),
         dXii_con=round((Xii_cfl/Xii_bln-1)*100,2),
         dXii_full=round((Xii_full/Xii_bln-1)*100,2),
         dPi_con=round(((Pi_i_cfl/Pi_i_bln)^(1/(1-sigma_ij))- 1)*100,2),
         dPi_full=round(((Pi_i_full/Pi_i_bln)^(1/(1-sigma_ij))- 1)*100,2))

Mfull<-ge_data%>%
  mutate(Destination=as.character(Destination))%>%
  group_by(Destination)%>%
  summarise(Ej_bln=round(sum(Xij_bln),0),
            Ej_full=round(sum(Xij_full),0),
            Mi_bln=round(sum(Xij_bln * (Pair != "0-INTRA")),0),
            Mi_cfl=round(sum(Xij_cfl * (Pair != "0-INTRA")),0),
            Mi_full=round(sum(Xij_full * (Pair != "0-INTRA")),0),
            Rho_j_bln=max(Rho_j),
            Rho_j_cfl=max(Rho_j_cfl),
            Rho_j_full=max(Rho_j_cfl),
            d_p_j_cfl=round((max(d_p_j)-1)*100,2),
            d_p_j_full=round((max(d_p_j_full)-1)*100,2))%>%
  mutate(dEj_full=round((Ej_full/Ej_bln-1)*100,2),
         dMi_con=round((Mi_cfl/Mi_bln-1)*100,2),
         dMi_full=round((Mi_full/Mi_bln-1)*100,2),
         dRho_con=round(((Rho_j_cfl/Rho_j_bln)^(1/(1-sigma_ij))- 1)*100,2),
         dRho_full=round(((Rho_j_full/Rho_j_bln)^(1/(1-sigma_ij))- 1)*100,2))

Output<-merge(x=Xfull,y=Mfull,by.x=c("Origin"),by.y=c("Destination"),all.x=TRUE)%>%
  mutate(dW_con=round((((Xii_cfl/Ej_bln)/(Xii_bln/Ej_bln))^(1/(1-sigma_ij))-1)*100,2),
         dW_full=round((((Xii_cfl/Ej_bln)/(Xii_bln/Ej_bln))^(1/(1-sigma_ij))-1)*100,2),
         RGDP_bln = Yi_bln / (Rho_j_bln^(1 / (1 - sigma_ij))),
         RGDP_full = Yi_full / (Rho_j_full^(1 / (1 - sigma_ij))),
         dRGDP = round((RGDP_full/RGDP_bln - 1) * 100,2))%>%
 dplyr:: select(-Pi_i_bln,-Pi_i_cfl,-Pi_i_full,-Rho_j_bln,-Rho_j_cfl,-Rho_j_full)


global<-Output%>%
  summarise(d_p_i=weighted.mean(d_p_i_full,Yi_bln),
            d_p_j=weighted.mean(d_p_j_full,Ej_bln),
            Yi_bln=sum(Yi_bln),Yi_full=sum(Yi_full),dYi=round((sum(Yi_full)/sum(Yi_bln)-1)*100,2),
            Xi_bln=sum(Xi_bln),Xi_full=sum(Xi_full),dXi=round((sum(Xi_full)/sum(Yi_bln)-1)*100,2))

col_order<-c("Origin","Yi_bln","Ej_bln","Xi_bln","Mi_bln","Xii_bln",
             "Xi_cfl","dXi_con","Mi_cfl","dMi_con","Xii_cfl","dXii_con","dPi_con","dRho_con","d_p_i_cfl","d_p_j_cfl","dW_con",
             "Yi_full","dYi_full","Ej_full","dEj_full","Xi_full","dXi_full","Mi_full","dMi_full","Xii_full","dXii_full",
             "dPi_full","dRho_full","d_p_i_full","d_p_j_full",
             "dW_full","dRGDP")

Output<-Output[,col_order]

colnames(Output)<-c("Country","Production","Expenditure","Exports","Imports","Domestic trade",
                    "Exports, conditional","Change in exports, conditional (%)",
                    "Imports, conditional","Change in imports, conditional (%)",
                    "Domestic trade, conditional","Change in domestic trade, conditional (%)",
                    "Change in outward mulilateral resistance, conditional (%)",
                    "Change in inward mulilateral resistance, conditional (%)",
                    "Change in output prices, conditional (%)",
                    "Change in input prices, conditional (%)",
                    "Change in consumer welfare, conditional (%)",
                    "Production, full effect","Change in production, full effect (%)",
                    "Expenditure, full effect","Change in expenditure, full effect (%)",
                    "Exports, full effect","Change in exports, full effect (%)",
                    "Imports, full effect","Change in imports, full effect (%)",
                    "Domestic trade, full effect","Change in domestic trade, full effect (%)",
                    "Change in outward mulilateral resistance, full effect (%)",
                    "Change in inward mulilateral resistance, full effect (%)",
                    "Change in output prices, full effect (%)",
                    "Change in input prices, full effect (%)",
                    "Change in consumer welfare, full effect (%)",
                    "Change in real GDP, full effect (%)")

write.csv(Output,paste0(table_fn,PROD,"_GE_effects_",SCENARIO,".csv"))

colnames(global)<-c("Change in producer prices (%)","Change in consumer prices (%)",
                    "Output, baseline","Output, counterfactual","Change in output (%)",
                    "Trade, baseline","Trade, counterfactual","Change in trade (%)")

write.csv(global,paste0(table_fn,PROD,"GLOBAL_GE_effects_",SCENARIO,".csv"))

summary_by_iso<-summary_by_iso<-dplyr::select(ge_data,Origin,Destination,Year,Trade,Ej_full,Ej,Yi_full,Yi,Xi_full,Xi_bln,d_p_i_full,d_p_j_full,Xij_full,Xij_cfl,Xij_bln)

write.csv(summary_by_iso,paste0(table_fn,PROD,"_Summary_GE_by_iso_",SCENARIO,".csv"))

