# Function to get policy effects, args are a/the model, model formula components
get_policy_effects<-function(mdl=bln_mdl,mdl_data=filter(df,DirectionTrade>0),fml_lhs=lhs,fml_rhs=rhs,
  DS_vars = "DAR|DBR|DGR|RAR|RBR|RGR|LAR|LBR|LGR|AAR|AGR"){
  # Get X (predictors)
  mf<-model.frame(as.formula(paste0(fml_lhs,fml_rhs,("-1"))),data=mdl_data) # model.frame, transformed variables
  mt <- attr(x = mf, which = "terms") # model specs
  X <- model.matrix(object = mt, data = mf) # X, predictors with interactions
  X<-X[,!colnames(X)%in%bln_mdl$collin.var] # Drop collinear variables
  # Index for merging
  idx<-mdl_data[,c("Origin","Destination","Year")]
  # Coefficients dataframe
  df_c<-tibble::rownames_to_column(as.data.frame(mdl$coefficients), "Coef") # Add rownames for coefficient as a variable
  colnames(df_c)<-c("Variable","Coef")
  # Effects
  temp<-sweep(X,MARGIN=2,df_c$Coef,"*")
  colnames(temp)<-paste0("ln_tij_",colnames(temp))
  temp<-as.data.frame(temp)
  # Total policy effects
  temp$ln_tij_Policies<-rowSums(temp)
  # Total tariff effects, if in data 
  if(any(grepl("Tariff",colnames(temp)))){
    temp$ln_tij_Tariff_total<-rowSums(as.data.frame(temp[,grepl("Tariff",colnames(temp))]))
  }
  # Total tariff direct effects, if in data 
  if(any(grepl("Tariff",colnames(temp)))){
    temp$ln_tij_Tariff_direct<-rowSums(temp[,(grepl("Tariff",colnames(temp)))&(!grepl(":",colnames(temp)))])
  }
  # Total domestic support (if in data)
  if(any(grepl(DS_vars,colnames(temp)))){
    temp$ln_tij_DS_total<-rowSums(temp[,grepl(DS_vars,colnames(temp))])
  }
  # Total domestic support direct effects (if in data)
  if(any(grepl(DS_vars,colnames(temp)))){
    temp$ln_tij_DS_direct<-rowSums(temp[,(grepl(DS_vars,colnames(temp)))&(!grepl(":",colnames(temp)))])
  }
  # Total NTM effects if in data
  if(any(grepl("IMP_CON|IMP_REG|IMP_RES|IMP_PRO|IMP_PRI|IMP_COM|IMP_QTY|EXP_RES|EXP_SUB|EXP_PRI",colnames(temp)))){
    temp$ln_tij_NTM_total<-rowSums(temp[,grepl("IMP_CON|IMP_REG|IMP_RES|IMP_PRO|IMP_PRI|IMP_COM|IMP_QTY|EXP_RES|EXP_SUB|EXP_PRI",colnames(temp))])
  }
  
  effects<-bind_cols(idx,temp)
  return(effects)
}

# Get policy effects from glmnet coeficient information
get_policy_effects_glmnet<-function(Xall=df,coefs=policy_coefs,DS_vars = "DAR|DBR|DGR|RAR|RBR|RGR|LAR|LBR|LGR|AAR|AGR"){
    # Model vars
    coef_names<-coefs$name

    # Vars
    Xp<-Xall[,coef_names]

    # Index for merging
    idx<-Xall[,c("Origin","Destination","Year")]

    # Calculate effects of each policy
    temp<-sweep(Xp,MARGIN=2,coefs$coef,"*")

    # Effects
    colnames(temp)<-paste0("ln_tij_",colnames(temp))
    temp<-as.data.frame(temp)
    # Total policy effects
    temp$ln_tij_Policies<-rowSums(temp)
    # Total tariff effects, if in data 
    if(any(grepl("Tariff",colnames(temp)))){
      temp$ln_tij_Tariff_total<-rowSums(as.data.frame(temp[,grepl("Tariff",colnames(temp))]))
    }
    # Total tariff direct effects, if in data 
    if(any(grepl("Tariff",colnames(temp)))){
      temp$ln_tij_Tariff_direct<-rowSums(temp[,(grepl("Tariff",colnames(temp)))&(!grepl(":",colnames(temp)))])
    }
    # Total domestic support (if in data)
    if(any(grepl(DS_vars,colnames(temp)))){
      temp$ln_tij_DS_total<-rowSums(temp[,grepl(DS_vars,colnames(temp))])
    }
    # Total domestic support direct effects (if in data) excluding any interactions
    #if(any(grepl("DAR|DBR|DGR|RAR|RBR|RGR|LAR|LBR|LGR|AAR|AGR",colnames(temp)))){
    #  temp$ln_tij_DS_direct<-rowSums(temp[,(grepl("DAR|DBR|DGR|RAR|RBR|RGR|LAR|LBR|LGR|AAR|AGR",colnames(temp)))&(!grepl(":|EU",colnames(temp)))])
    #}
    # Total NTM effects if in data
    if(any(grepl("IMP_CON|IMP_REG|IMP_RES|IMP_PRO|IMP_PRI|IMP_COM|IMP_QTY|EXP_RES|EXP_SUB|EXP_PRI",colnames(temp)))){
      temp$ln_tij_NTM_total<-rowSums(temp[,grepl("IMP_CON|IMP_REG|IMP_RES|IMP_PRO|IMP_PRI|IMP_COM|IMP_QTY|EXP_RES|EXP_SUB|EXP_PRI",colnames(temp))])
    }

    effects<-bind_cols(idx,temp)
    return(effects)
}

# Function to get marginal effects where the model includes interaction terms
get_marginal_effects<-function(mdl=bln_mdl,mdl_data=filter(df,DirectionTrade>0),fml_lhs=lhs,fml_rhs=rhs){
  # Get X (predictors)
  mf<-model.frame(as.formula(paste0(fml_lhs,fml_rhs,("-1"))),data=mdl_data) # model.frame, transformed variables
  mt <- attr(x = mf, which = "terms") # model specs
  X <- model.matrix(object = mt, data = mf) # X, predictors with interactions
  # Index for merging
  idx<-mdl_data[,c("Origin","Destination","Year")]
  # Coefficients dataframe
  df_c<-tibble::rownames_to_column(as.data.frame(mdl$coefficients), "Coef") # Add rownames for coefficient as a variable
  colnames(df_c)<-c("Variable","Coef")
  # Marginal effects
  # Check if there are any interaction terms (the term/name int here is used to denote interaction)
  if(!any(grepl(":",colnames(X)))){
    stop("No interaction terms")
  }
  #ints<-colnames(X)[grepl(":",colnames(X))]
  ints<-df_c$Variable[grepl(":",df_c$Variable)]
  in_coefs<-c(unlist(str_split(df_c$Variable,":")),ints)
  # Check if collinear variables remain in coefficients
  X<-X[,colnames(X)%in%in_coefs] # Drop collinear variables
  n_ints<-length(ints)
  marg_vars<-unique(unlist(str_split(ints,":"))) # Variables for which marginal effects need calculating
  int_calc_list<-list() # List to collect all/each marginal effect in a way that enables aggregation
  int_list<-list() # List that collects marginal effects with reference to the interaction term (first var is the one for which the effect is calculated)
  for(i in 1:n_ints){
    get_int<-ints[i]
    split_int<-unlist(str_split(get_int,":"))
    intX = as.data.frame(X[,colnames(X)%in%split_int])
    single<-ncol(intX)==1 # Interaction excludes full second effect
    single_name=split_int[split_int%in%colnames(X)]
    if(single){colnames(intX)<-single_name}
    int_c<-df_c[df_c$Variable==get_int,]$Coef
    int_c_vars<-unlist(str_split(df_c[df_c$Variable==get_int,]$Variable,":"))
    if(ncol(intX)>1){intX = intX[,int_c_vars]} # Make sure the data is in the same order as the coefficients
    intX<-cbind(idx,intX)
    direct_c1<-df_c[df_c$Variable==split_int[1],]$Coef
    direct_c2<-df_c[df_c$Variable==split_int[2],]$Coef
    if(sum(direct_c1)==0&!single){direct_c1<-numeric(nrow(intX))}
    if(sum(direct_c2)==0&!single){direct_c2<-numeric(nrow(intX))}
    if(!single){
    ME<-data.frame(First=as.vector(intX[,int_c_vars[2]])*int_c+direct_c1,
                   Second=as.vector(intX[,int_c_vars[1]])*int_c+direct_c2)
    colnames(ME)<-int_c_vars
    }else{
      ME<-data.frame(First=as.vector(intX[,single_name])*int_c+direct_c1,
                     Second=as.vector(intX[,int_c_vars[1]])*int_c+direct_c2)
    }
    int_calc_list[[i]]<-ME
    colnames(ME)<-c(paste0("MARG_",int_c_vars[1],":",int_c_vars[2]),paste0("MARG_",int_c_vars[2],":",int_c_vars[1]))
    int_list[[i]]<-ME
  }
  # Marginal effects by interaction terms/components
  int_ME<-bind_cols(int_list)
  
  # Values for calculations of overall variable marginal effects
  int_ME_calcs<-as.data.frame(bind_cols(int_calc_list))
  vars_in_calcs<-colnames(int_ME_calcs)
  var_order<-as.character(stringr::str_replace_all(string = as.character(gsub("\\..*","",as.character(vars_in_calcs))), pattern = "[^A-Za-z0-9-\\(-\\_-\\)]", replacement = ""))
  
  ME_total_list<-list()  
  for(i in seq_along(marg_vars)){
    temp_calcs<-as.data.frame(int_ME_calcs[,var_order%in%marg_vars[i]])
    temp_calcs$Sum<-rowSums(temp_calcs)
    temp_calcs<-select(temp_calcs,Sum)
    colnames(temp_calcs)<-marg_vars[i]
    ME_total_list[[i]]<-temp_calcs
  }
  # Marginal effects by variables - for those with interaction effects
  ME_by_var<-bind_cols(ME_total_list)
  
  effects<-bind_cols(idx,ME_by_var,int_ME)
  return(effects)
  
  
  
}
