hdfeppml_corrected<-function (y, x = NULL, fes = NULL, tol = 1e-08, hdfetol = 1e-04, 
                              mu = NULL, saveX = TRUE, colcheck = TRUE, colcheck_x = colcheck, 
                              colcheck_x_fes = colcheck, init_z = NULL, verbose = FALSE, 
                              maxiter = 1000, cluster = NULL, vcv = TRUE) 
{
  if (missing(x) & missing(fes)) {
    stop("Please provide at least one of the arguments x or fes.")
  }
  if (!missing(x)) {
    x <- data.matrix(x)
  }
  n <- length(y)
  crit <- 1
  iter <- 0
  old_deviance <- 0
  if (!missing(x)) {
    include_x <- 1:ncol(x)
    b <- matrix(NA, nrow = ncol(x), ncol = 1)
    xnames <- colnames(x)
    if (colcheck_x == TRUE | colcheck_x_fes == TRUE) {
      if (verbose == TRUE) {
        print("checking collinearity")
      }
      if (!missing(fes)) {
        include_x <- penppml:::collinearity_check(y = y, x = x, 
                                        fes = fes, hdfetol = 1e-06, colcheck_x = colcheck_x, 
                                        colcheck_x_fes = colcheck_x_fes)
        x <- x[, include_x]
      }
      else {
        print("do collinearity check")
        include_x <- penppml:::collinearity_check(y = y, x = x, 
                                        hdfetol = 1e-06, colcheck_x = colcheck_x, colcheck_x_fes = FALSE)
        x <- x[, include_x]
      }
    }
  }
  if (verbose == TRUE) {
    print("beginning estimation")
  }
  while (crit > tol & iter < maxiter) {
    iter <- iter + 1
    if (verbose == TRUE) {
      print(iter)
    }
    if (iter == 1) {
      if (is.null(mu)) 
        mu <- (y + mean(y))/2
      z <- (y - mu)/mu + log(mu)
      eta <- log(mu)
      last_z <- z
      if (is.null(init_z)) {
        reg_z <- matrix(z)
      }
      else {
        reg_z <- init_z
      }
      if (!missing(x)) {
        reg_x <- x
      }
    }
    else {
      last_z <- z
      z <- (y - mu)/mu + log(mu)
      reg_z <- matrix(z - last_z + z_resid)
      if (!missing(x)) {
        reg_x <- x_resid
      }
    }
    if (verbose == TRUE) {
      print("within transformation step")
    }
    if (!missing(fes)) {
      if (is.null(fes)) {
        z_resid <- collapse::fwithin(x = reg_z, g = factor(rep(1, 
                                                               length(reg_z))), w = mu)
        if (!missing(x)) {
          x_resid <- collapse::fwithin(x = reg_x, g = factor(rep(1, 
                                                                 length(reg_z))), w = mu)
        }
      }
      else {
        z_resid <- collapse::fhdwithin(reg_z, fes, w = mu)
        if (!missing(x)) {
          x_resid <- collapse::fhdwithin(reg_x, fes, 
                                         w = mu)
        }
      }
    }
    else {
      z_resid <- reg_z
      if (!missing(x)) {
        x_resid <- reg_x
      }
    }
    if (verbose == TRUE) {
      print("obtaining coefficients")
    }
    if (verbose == TRUE) {
      print(iter)
    }
    if (!missing(x)) {
      reg <- penppml:::fastolsCpp(sqrt(mu) * x_resid, sqrt(mu) * 
                          z_resid)
      b[include_x] <- reg
      reg <- list(coefficients = b)
      if (length(include_x) == 1) {
        reg$residuals <- z_resid - x_resid * b[include_x]
      }
      else {
        reg$residuals <- z_resid - x_resid %*% b[include_x]
      }
    }
    else {
      reg <- NULL
      reg$residuals <- z_resid
    }
    mu <- as.numeric(exp(z - reg$residuals))
    mu[which(mu < 1e-190)] <- 1e-190
    mu[mu > 1e+190] <- 1e+190
    if (verbose == TRUE) {
      print("info on residuals")
      print(max(reg$residuals))
      print(min(reg$residuals))
      print("info on means")
      print(max(mu))
      print(min(mu))
      print("info on coefficients")
      print(max(b[include_x]))
      print(min(b[include_x]))
    }
    if (verbose == TRUE) {
      print("calculating deviance")
    }
    temp <- -(y * log(y/mu) - (y - mu))
    temp[which(y == 0)] <- -mu[which(y == 0)]
    deviance <- -2 * sum(temp)/n
    if (deviance < 0 | is.na(deviance)){
      deviance = 0
      if(is.na(deviance)){print("deviance is NA")}
    }
    
    delta_deviance <- old_deviance - deviance
    if (!is.na(delta_deviance) & (deviance < 0.1 * delta_deviance)) {
      delta_deviance <- deviance
    }
    if (verbose == TRUE) {
      print("checking critical value")
    }
    denom_crit = max(c(min(c(deviance, old_deviance)), 0.1))
    crit = abs(delta_deviance)/denom_crit
    if (verbose == TRUE) {
      print(deviance)
      print(crit)
    }
    old_deviance <- deviance
  }
  temp <- -(y * log(y/mu) - (y - mu))
  temp[which(y == 0)] <- 0
  if (verbose == TRUE) {
    print("converged")
  }
  if (!missing(x)) {
    k <- ncol(matrix(x))
    n <- length(y)
    reg$mu <- mu
    reg$deviance <- -2 * sum(temp)/n
    reg$bic <- deviance + k * log(n)/n
    rownames(reg$coefficients) <- xnames
  }
  else {
    reg$mu <- mu
  }
  if (saveX == TRUE) {
    if (!missing(x)) {
      reg[["x_resid"]] <- x_resid
    }
    reg[["z_resid"]] <- z_resid
  }
  if (!missing(x)) {
    if (vcv) {
      if (!is.null(cluster)) {
        nclusters <- nlevels(droplevels(cluster, exclude = if (anyNA(levels(cluster))) 
          NULL
          else NA))
        het_matrix <- (1/nclusters) * cluster_matrix((y - 
                                                        mu)/sum(sqrt(mu)), cluster, x_resid)
        W <- (1/nclusters) * (t(mu * x_resid) %*% x_resid)/sum(sqrt(mu))
        R <- try(chol(W), silent = FALSE)
        V <- (1/nclusters) * chol2inv(R) %*% het_matrix %*% 
          chol2inv(R)
        V <- nclusters/(nclusters - 1) * V
      }
      else {
        e = y - mu
        het_matrix = (1/n) * t(x_resid * e) %*% (x_resid * 
                                                   e)
        W = (1/n) * (t(mu * x_resid) %*% x_resid)
        R = try(chol(W), silent = TRUE)
        V = (1/n) * chol2inv(R) %*% het_matrix %*% chol2inv(R)
        V = (n/(n - 1)) * V
      }
    }
    reg[["se"]] <- sqrt(diag(V))
  }
  return(reg)
}