# CALCULATE CHANGES IN COSTS GIVEN POLICY SHOCKS
# Note the 'positive only' term means that the shock is only applied to instances where PSE is positive

get_policy_shocks<-function(mdl=bln_mdl,mdl_data=bln,
                            shock_vars_m = c("PSE_M","PSE_M_L3","SCT_M_L1"),
                            shock_vars_x = c("PSE_X","PSE_X_L3","SCT_X_L1"), 
                            shock_val = -0.10,
                            shock_country = "ALL",
                            positive_only=TRUE,
                            DS_vars = "DAR|DBR|DGR|RAR|RBR|RGR|LAR|LBR|LGR|AAR|AGR",
                            ref=REF){
  # Translate country groups - for now I only allow for EU or OECD as groups
  if(shock_country=="EUN"){
    country_list=c("AUT","BEL","BGR","HRV","CYP","CZE","DNK","EST","FIN","FRA","DEU","GRC","HUN","IRL","ITA","LVA","LTU","LUX","MLT","NLD","POL","PRT","ROU","SVK","SVN","ESP","SWE","GBR")
  } else if(shock_country=="OECD"){
    country_list=c("AUS","AUT","BEL","CAN","CHE","CHL","COL","CZE","DEU","DNK","ESP","EST","EUN","FIN","FRA","GBR","GRC","HUN","IRL","ISL","ISR","ITA","JPN","KOR","LTU","LUX","LVA","MEX","NLD","NOR","NZL","POL","PRT","SVK","SVN","SWE","TUR","USA")
  } else {country_list=shock_country}
  if(ref%in%country_list){country_list<-gsub(ref,paste0("0-",ref),country_list)}
  # Get X (predictors)
  lhs<-paste0(as.character(bln_mdl$fml[[2]])," ~ ")
  rhs<-as.character(Reduce(paste,deparse(bln_mdl[["fml"]][[3]])))
  mf<-model.frame(as.formula(paste0(lhs,rhs,("-1"))),data=mdl_data) # model.frame, transformed variables
  mt <- attr(x = mf, which = "terms") # model specs
  X <- model.matrix(object = mt, data = mf) # X, predictors with interactions
  X<-X[,!colnames(X)%in%mdl$collin.var] # Drop collinear variables
  # Index for merging
  idx<-mdl_data[,c("Origin","Destination","Year")]
  # Coefficients dataframe
  df_c<-tibble::rownames_to_column(as.data.frame(mdl$coefficients), "Coef") # Add rownames for coefficient as a variable
  colnames(df_c)<-c("Variable","Coef")
  # Add policy shocks to the X
  # Import shocks
  if(length(shock_vars_m)>0){
    positive_m=as.data.frame(X[,shock_vars_m]>0)
    if(shock_country=="ALL"&positive_only==TRUE){
        for(i in seq_along(shock_vars_m)){
          X[positive_m[,i],shock_vars_m[i]]<-X[positive_m[,i],shock_vars_m[i]]*(1+shock_val)
        }
    }else if(positive_only==TRUE){
      for(i in seq_along(shock_vars_m)){
        X[positive_m[,i]&idx$Destination%in%country_list,shock_vars_m[i]]<-X[positive_m[,i]&idx$Destination%in%country_list,shock_vars_m[i]]*(1+shock_val)
      }
    }else{
      X[idx$Destination%in%country_list,shock_vars_m]<-X[idx$Destination%in%country_list,shock_vars_m]*(1+shock_val)
    }
  }
  if(length(shock_vars_x)>0){
    positive_x=as.data.frame(X[,shock_vars_x]>0)
    if(shock_country=="ALL"&positive_only==TRUE){
      for(i in seq_along(shock_vars_x)){
        X[positive_x[,i],shock_vars_x[i]]<-X[positive_x[,i],shock_vars_x[i]]*(1+shock_val)
      }
    }else if(positive_only==TRUE){
      for(i in seq_along(shock_vars_x)){
        X[positive_x[,i]&idx$Origin%in%country_list,shock_vars_x[i]]<-X[positive_x[,i]&idx$Origin%in%country_list,shock_vars_x[i]]*(1+shock_val)
      }
    }else{
      X[idx$Origin%in%country_list,shock_vars_x]<-X[idx$Origin%in%country_list,shock_vars_x]*(1+shock_val)
    }
  }
  
  # Effects
  temp<-sweep(X,MARGIN=2,df_c$Coef,"*")
  colnames(temp)<-paste0("ln_tij_",colnames(temp))
  temp<-as.data.frame(temp)
  # Total policy effects
  temp$ln_tij_Policies<-rowSums(temp)
  # Total tariff direct effects, if in data 
  if(any(grepl("Tariff",colnames(temp)))){
    temp$ln_tij_Tariff_direct<-rowSums(as.data.frame(temp[,(grepl("Tariff",colnames(temp)))&(!grepl(":",colnames(temp)))]))
  }
  # Total domestic support (if in data)
  if(any(grepl(DS_vars,colnames(temp)))){
    temp$ln_tij_DS_total<-rowSums(temp[,grepl(DS_vars,colnames(temp))])
  }
  # Total domestic support direct effects (if in data)
  if(any(grepl(DS_vars,colnames(temp)))){
    temp$ln_tij_DS_direct<-rowSums(temp[,(grepl(DS_vars,colnames(temp)))&(!grepl(":",colnames(temp)))])
  }
  # Total NTM effects if in data
  if(any(grepl("IMP_CON|IMP_REG|IMP_RES|IMP_PRO|IMP_PRI|IMP_COM|IMP_QTY|EXP_RES|EXP_SUB|EXP_PRI",colnames(temp)))){
    temp$ln_tij_NTM_total<-rowSums(temp[,grepl("IMP_CON|IMP_REG|IMP_RES|IMP_PRO|IMP_PRI|IMP_COM|IMP_QTY|EXP_RES|EXP_SUB|EXP_PRI",colnames(temp))])
  }
  
  effects<-bind_cols(idx,temp)
  return(effects)
}


    
get_policy_shocks_glmnet<-function(coefs=policy_coefs,mdl_data=bln,
                            shock_vars_m = c("PSE_M","PSE_M_L3","SCT_M_L1"),
                            shock_vars_x = c("PSE_X","PSE_X_L3","SCT_X_L1"), 
                            shock_val = -0.10,
                            shock_country = "ALL",
                            positive_only=TRUE,
                            DS_vars = "DAR|DBR|DGR|RAR|RBR|RGR|LAR|LBR|LGR|AAR|AGR",
                            ref=REF){
  # Translate country groups - for now I only allow for EU or OECD as groups
  if(shock_country=="EUN"){
    country_list=c("AUT","BEL","BGR","HRV","CYP","CZE","DNK","EST","FIN","FRA","DEU","GRC","HUN","IRL","ITA","LVA","LTU","LUX","MLT","NLD","POL","PRT","ROU","SVK","SVN","ESP","SWE","GBR")
  } else if(shock_country=="OECD"){
    country_list=c("AUS","AUT","BEL","CAN","CHE","CHL","COL","CZE","DEU","DNK","ESP","EST","EUN","FIN","FRA","GBR","GRC","HUN","IRL","ISL","ISR","ITA","JPN","KOR","LTU","LUX","LVA","MEX","NLD","NOR","NZL","POL","PRT","SVK","SVN","SWE","TUR","USA")
  } else {country_list=shock_country}
  if(ref%in%country_list){country_list<-gsub(ref,paste0("0-",ref),country_list)}
  # Get X (predictors)
  # Model vars
  coef_names<-coefs$name
  X<-mdl_data[,coef_names]
  # Index for merging
  idx<-mdl_data[,c("Origin","Destination","Year")]
  # Coefficients dataframe
  df_c<-coefs
  colnames(df_c)<-c("Variable","Coef")
  # Add policy shocks to the X
  # Import shocks
  if(length(shock_vars_m)>0){
    positive_m=as.data.frame(X[,shock_vars_m]>0)
    if(shock_country=="ALL"&positive_only==TRUE){
        for(i in seq_along(shock_vars_m)){
          X[positive_m[,i],shock_vars_m[i]]<-X[positive_m[,i],shock_vars_m[i]]*(1+shock_val)
        }
    }else if(positive_only==TRUE){
      for(i in seq_along(shock_vars_m)){
        X[positive_m[,i]&idx$Destination%in%country_list,shock_vars_m[i]]<-X[positive_m[,i]&idx$Destination%in%country_list,shock_vars_m[i]]*(1+shock_val)
      }
    }else{
      X[idx$Destination%in%country_list,shock_vars_m]<-X[idx$Destination%in%country_list,shock_vars_m]*(1+shock_val)
    }
  }
  if(length(shock_vars_x)>0){
    positive_x=as.data.frame(X[,shock_vars_x]>0)
    if(shock_country=="ALL"&positive_only==TRUE){
      for(i in seq_along(shock_vars_x)){
        X[positive_x[,i],shock_vars_x[i]]<-X[positive_x[,i],shock_vars_x[i]]*(1+shock_val)
      }
    }else if(positive_only==TRUE){
      for(i in seq_along(shock_vars_x)){
        X[positive_x[,i]&idx$Origin%in%country_list,shock_vars_x[i]]<-X[positive_x[,i]&idx$Origin%in%country_list,shock_vars_x[i]]*(1+shock_val)
      }
    }else{
      X[idx$Origin%in%country_list,shock_vars_x]<-X[idx$Origin%in%country_list,shock_vars_x]*(1+shock_val)
    }
  }
  
  # Effects
  temp<-sweep(X,MARGIN=2,df_c$Coef,"*")
  colnames(temp)<-paste0("ln_tij_",colnames(temp))
  temp<-as.data.frame(temp)
  # Total policy effects
  temp$ln_tij_Policies<-rowSums(temp)
  # Total tariff direct effects, if in data 
  if(any(grepl("Tariff",colnames(temp)))){
    temp$ln_tij_Tariff_direct<-rowSums(as.data.frame(temp[,(grepl("Tariff",colnames(temp)))&(!grepl(":",colnames(temp)))]))
  }
  # Total domestic support (if in data)
  if(any(grepl(DS_vars,colnames(temp)))){
    temp$ln_tij_DS_total<-rowSums(temp[,grepl(DS_vars,colnames(temp))])
  }
  # Total domestic support direct effects (if in data)
  if(any(grepl(DS_vars,colnames(temp)))){
    temp$ln_tij_DS_direct<-rowSums(temp[,(grepl(DS_vars,colnames(temp)))&(!grepl(":",colnames(temp)))])
  }
  # Total NTM effects if in data
  if(any(grepl("IMP_CON|IMP_REG|IMP_RES|IMP_PRO|IMP_PRI|IMP_COM|IMP_QTY|EXP_RES|EXP_SUB|EXP_PRI",colnames(temp)))){
    temp$ln_tij_NTM_total<-rowSums(temp[,grepl("IMP_CON|IMP_REG|IMP_RES|IMP_PRO|IMP_PRI|IMP_COM|IMP_QTY|EXP_RES|EXP_SUB|EXP_PRI",colnames(temp))])
  }
  
  effects<-bind_cols(idx,temp)
  return(effects)
}

get_policy_shocks_glmnet_shift2green<-function(coefs=policy_coefs,mdl_data=bln_g,
                                   shock_vars_m = c("PSE_M","PSE_M_L3","SCT_M_L1"),
                                   shock_vars_x = c("PSE_X","PSE_X_L3","SCT_X_L1"), 
                                   shock_val = -0.10,
                                   shock_country = "ALL",
                                   positive_only=TRUE,
                                   ref=REF){
  # Translate country groups - for now I only allow for EU or OECD as groups
  if(shock_country=="EUN"){
    country_list=c("AUT","BEL","BGR","HRV","CYP","CZE","DNK","EST","FIN","FRA","DEU","GRC","HUN","IRL","ITA","LVA","LTU","LUX","MLT","NLD","POL","PRT","ROU","SVK","SVN","ESP","SWE","GBR")
  } else if(shock_country=="OECD"){
    country_list=c("AUS","AUT","BEL","CAN","CHE","CHL","COL","CZE","DEU","DNK","ESP","EST","EUN","FIN","FRA","GBR","GRC","HUN","IRL","ISL","ISR","ITA","JPN","KOR","LTU","LUX","LVA","MEX","NLD","NOR","NZL","POL","PRT","SVK","SVN","SWE","TUR","USA")
  } else {country_list=shock_country}
  if(ref%in%country_list){country_list<-gsub(ref,paste0("0-",ref),country_list)}
  # Recalculate green box
  mdl_data<-mdl_data%>%
    mutate(# Export direction
      AGR_X=ifelse(Origin%in%country_list,AGR_X*dAGR_X,AGR_X),
      RGR_X=ifelse(Origin%in%country_list,RGR_X*dRGR_X,RGR_X),
      DGR_X=ifelse(Origin%in%country_list,DGR_X*dDGR_X,DGR_X),
      LGR_X=ifelse(Origin%in%country_list,LGR_X*dLGR_X,LGR_X),
      TGR_X=ifelse(Origin%in%country_list,TGR_X*dTGR_X,TGR_X),
      # First lag
      AGR_X_L1=ifelse(Origin%in%country_list,AGR_X_L1*dAGR_L1_X,AGR_X_L1),
      RGR_X_L1=ifelse(Origin%in%country_list,RGR_X_L1*dRGR_L1_X,RGR_X_L1),
      DGR_X_L1=ifelse(Origin%in%country_list,DGR_X_L1*dDGR_L1_X,DGR_X_L1),
      LGR_X_L1=ifelse(Origin%in%country_list,LGR_X_L1*dLGR_L1_X,LGR_X_L1),
      TGR_X_L1=ifelse(Origin%in%country_list,TGR_X_L1*dTGR_L1_X,TGR_X_L1),
      # Second lag
      AGR_X_L2=ifelse(Origin%in%country_list,AGR_X_L2*dAGR_L2_X,AGR_X_L2),
      RGR_X_L2=ifelse(Origin%in%country_list,RGR_X_L2*dRGR_L2_X,RGR_X_L2),
      DGR_X_L2=ifelse(Origin%in%country_list,DGR_X_L2*dDGR_L2_X,DGR_X_L2),
      LGR_X_L2=ifelse(Origin%in%country_list,LGR_X_L2*dLGR_L2_X,LGR_X_L2),
      TGR_X_L2=ifelse(Origin%in%country_list,TGR_X_L2*dTGR_L2_X,TGR_X_L2),
      # Third lag
      AGR_X_L3=ifelse(Origin%in%country_list,AGR_X_L3*dAGR_L3_X,AGR_X_L3),
      RGR_X_L3=ifelse(Origin%in%country_list,RGR_X_L3*dRGR_L3_X,RGR_X_L3),
      DGR_X_L3=ifelse(Origin%in%country_list,DGR_X_L3*dDGR_L3_X,DGR_X_L3),
      LGR_X_L3=ifelse(Origin%in%country_list,LGR_X_L3*dLGR_L3_X,LGR_X_L3),
      TGR_X_L3=ifelse(Origin%in%country_list,TGR_X_L3*dTGR_L3_X,TGR_X_L3),
      # Import direction
      AGR_M=ifelse(Destination%in%country_list,AGR_M*dAGR_M,AGR_M),
      RGR_M=ifelse(Destination%in%country_list,RGR_M*dRGR_M,RGR_M),
      DGR_M=ifelse(Destination%in%country_list,DGR_M*dDGR_M,DGR_M),
      LGR_M=ifelse(Destination%in%country_list,LGR_M*dLGR_M,LGR_M),
      TGR_M=ifelse(Destination%in%country_list,TGR_M*dTGR_M,TGR_M),
      # First lag
      AGR_M_L1=ifelse(Destination%in%country_list,AGR_M_L1*dAGR_L1_M,AGR_M_L1),
      RGR_M_L1=ifelse(Destination%in%country_list,RGR_M_L1*dRGR_L1_M,RGR_M_L1),
      DGR_M_L1=ifelse(Destination%in%country_list,DGR_M_L1*dDGR_L1_M,DGR_M_L1),
      LGR_M_L1=ifelse(Destination%in%country_list,LGR_M_L1*dLGR_L1_M,LGR_M_L1),
      TGR_M_L1=ifelse(Destination%in%country_list,TGR_M_L1*dTGR_L1_M,TGR_M_L1),
      # Second lag
      AGR_M_L2=ifelse(Destination%in%country_list,AGR_M_L2*dAGR_L2_M,AGR_M_L2),
      RGR_M_L2=ifelse(Destination%in%country_list,RGR_M_L2*dRGR_L2_M,RGR_M_L2),
      DGR_M_L2=ifelse(Destination%in%country_list,DGR_M_L2*dDGR_L2_M,DGR_M_L2),
      LGR_M_L2=ifelse(Destination%in%country_list,LGR_M_L2*dLGR_L2_M,LGR_M_L2),
      TGR_M_L2=ifelse(Destination%in%country_list,TGR_M_L2*dTGR_L2_M,TGR_M_L2),
      # Third lag
      AGR_M_L3=ifelse(Destination%in%country_list,AGR_M_L3*dAGR_L3_M,AGR_M_L3),
      RGR_M_L3=ifelse(Destination%in%country_list,RGR_M_L3*dRGR_L3_M,RGR_M_L3),
      DGR_M_L3=ifelse(Destination%in%country_list,DGR_M_L3*dDGR_L3_M,DGR_M_L3),
      LGR_M_L3=ifelse(Destination%in%country_list,LGR_M_L3*dLGR_L3_M,LGR_M_L3),
      TGR_M_L3=ifelse(Destination%in%country_list,TGR_M_L3*dTGR_L3_M,TGR_M_L3),
      # EU interactions
      TGR_X_EU=ifelse(Origin%in%country_list,TGR_X_EU*dTGR_X,TGR_X_EU),
      TGR_X_EU_L1=ifelse(Origin%in%country_list,TGR_X_EU_L1*dTGR_L1_X,TGR_X_EU_L1),
      TGR_X_EU_L2=ifelse(Origin%in%country_list,TGR_X_EU_L2*dTGR_L2_X,TGR_X_EU_L2),
      TGR_X_EU_L3=ifelse(Origin%in%country_list,TGR_X_EU_L3*dTGR_L3_X,TGR_X_EU_L3))
      
      
  # Get X (predictors)
  # Index for merging
  idx<-mdl_data[,c("Origin","Destination","Year")]
  # Model vars
  coef_names<-coefs$name
  X<-mdl_data[,coef_names]
  
  # Coefficients dataframe
  df_c<-coefs
  colnames(df_c)<-c("Variable","Coef")
  # Add policy shocks to the X
  # Import shocks
  if(length(shock_vars_m)>0){
    positive_m=as.data.frame(X[,shock_vars_m]>0)
    if(shock_country=="ALL"&positive_only==TRUE){
      for(i in seq_along(shock_vars_m)){
        X[positive_m[,i],shock_vars_m[i]]<-X[positive_m[,i],shock_vars_m[i]]*(1+shock_val)
      }
    }else if(positive_only==TRUE){
      for(i in seq_along(shock_vars_m)){
        X[positive_m[,i]&idx$Destination%in%country_list,shock_vars_m[i]]<-X[positive_m[,i]&idx$Destination%in%country_list,shock_vars_m[i]]*(1+shock_val)
      }
    }else{
      X[idx$Destination%in%country_list,shock_vars_m]<-X[idx$Destination%in%country_list,shock_vars_m]*(1+shock_val)
    }
  }
  if(length(shock_vars_x)>0){
    positive_x=as.data.frame(X[,shock_vars_x]>0)
    if(shock_country=="ALL"&positive_only==TRUE){
      for(i in seq_along(shock_vars_x)){
        X[positive_x[,i],shock_vars_x[i]]<-X[positive_x[,i],shock_vars_x[i]]*(1+shock_val)
      }
    }else if(positive_only==TRUE){
      for(i in seq_along(shock_vars_x)){
        X[positive_x[,i]&idx$Origin%in%country_list,shock_vars_x[i]]<-X[positive_x[,i]&idx$Origin%in%country_list,shock_vars_x[i]]*(1+shock_val)
      }
    }else{
      X[idx$Origin%in%country_list,shock_vars_x]<-X[idx$Origin%in%country_list,shock_vars_x]*(1+shock_val)
    }
  }
  
  # Effects
  temp<-sweep(X,MARGIN=2,df_c$Coef,"*")
  colnames(temp)<-paste0("ln_tij_",colnames(temp))
  temp<-as.data.frame(temp)
  # Total policy effects
  temp$ln_tij_Policies<-rowSums(temp)
  # Total tariff direct effects, if in data 
  if(any(grepl("Tariff",colnames(temp)))){
    temp$ln_tij_Tariff_direct<-rowSums(as.data.frame(temp[,(grepl("Tariff",colnames(temp)))&(!grepl(":",colnames(temp)))]))
  }
  # Total domestic support (if in data)
  if(any(grepl("DAR|DBR|DGR|RAR|RBR|RGR|LAR|LBR|LGR|AAR|AGR",colnames(temp)))){
    temp$ln_tij_DS_total<-rowSums(temp[,grepl("DAR|DBR|DGR|RAR|RBR|RGR|LAR|LBR|LGR|AAR|AGR|TAR|TGR|TBR",colnames(temp))])
  }
  # Total domestic support direct effects (if in data)
  if(any(grepl("DAR|DBR|DGR|RAR|RBR|RGR|LAR|LBR|LGR|AAR|AGR",colnames(temp)))){
    temp$ln_tij_DS_direct<-rowSums(temp[,(grepl("DAR|DBR|DGR|RAR|RBR|RGR|LAR|LBR|LGR|AAR|AGR|TAR|TGR|TBR",colnames(temp)))&(!grepl(":",colnames(temp)))])
  }
  # Total NTM effects if in data
  if(any(grepl("IMP_CON|IMP_REG|IMP_RES|IMP_PRO|IMP_PRI|IMP_COM|IMP_QTY|EXP_RES|EXP_SUB|EXP_PRI",colnames(temp)))){
    temp$ln_tij_NTM_total<-rowSums(temp[,grepl("IMP_CON|IMP_REG|IMP_RES|IMP_PRO|IMP_PRI|IMP_COM|IMP_QTY|EXP_RES|EXP_SUB|EXP_PRI",colnames(temp))])
  }
  
  effects<-bind_cols(idx,temp)
  return(effects)
}
